try:
    import unittest2 as unittest
except ImportError:
    import unittest

import rope.base.exceptions
from rope.refactor import change_signature
from ropetest import testutils


class ChangeSignatureTest(unittest.TestCase):

    def setUp(self):
        super(ChangeSignatureTest, self).setUp()
        self.project = testutils.sample_project()
        self.pycore = self.project.pycore
        self.mod = testutils.create_module(self.project, 'mod')

    def tearDown(self):
        testutils.remove_project(self.project)
        super(ChangeSignatureTest, self).tearDown()

    def test_normalizing_parameters_for_trivial_case(self):
        code = 'def a_func():\n    pass\na_func()'
        self.mod.write(code)
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual(code, self.mod.read())

    def test_normalizing_parameters_for_trivial_case2(self):
        code = 'def a_func(param):\n    pass\na_func(2)'
        self.mod.write(code)
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual(code, self.mod.read())

    def test_normalizing_parameters_for_unneeded_keyword(self):
        self.mod.write('def a_func(param):\n    pass\na_func(param=1)')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual('def a_func(param):\n    pass\na_func(1)',
                          self.mod.read())

    def test_normalizing_parameters_for_unneeded_keyword_for_methods(self):
        code = 'class A(object):\n' \
               '    def a_func(self, param):\n' \
               '        pass\n' \
               'a_var = A()\n' \
               'a_var.a_func(param=1)\n'
        self.mod.write(code)
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        expected = 'class A(object):\n' \
                   '    def a_func(self, param):\n' \
                   '        pass\n' \
                   'a_var = A()\n' \
                   'a_var.a_func(1)\n'
        self.assertEqual(expected, self.mod.read())

    def test_normalizing_parameters_for_unsorted_keyword(self):
        self.mod.write('def a_func(p1, p2):\n    pass\na_func(p2=2, p1=1)')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual('def a_func(p1, p2):\n    pass\na_func(1, 2)',
                          self.mod.read())

    def test_raising_exceptions_for_non_functions(self):
        self.mod.write('a_var = 10')
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            change_signature.ChangeSignature(
                self.project, self.mod, self.mod.read().index('a_var') + 1)

    def test_normalizing_parameters_for_args_parameter(self):
        self.mod.write('def a_func(*arg):\n    pass\na_func(1, 2)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual('def a_func(*arg):\n    pass\na_func(1, 2)\n',
                          self.mod.read())

    def test_normalizing_parameters_for_args_parameter_and_keywords(self):
        self.mod.write(
            'def a_func(param, *args):\n    pass\na_func(*[1, 2, 3])\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual('def a_func(param, *args):\n    pass\n'
                          'a_func(*[1, 2, 3])\n', self.mod.read())

    def test_normalizing_functions_from_other_modules(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func(param):\n    pass\n')
        self.mod.write('import mod1\nmod1.a_func(param=1)\n')
        signature = change_signature.ChangeSignature(
            self.project, mod1, mod1.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual('import mod1\nmod1.a_func(1)\n', self.mod.read())

    def test_normalizing_parameters_for_keyword_parameters(self):
        self.mod.write('def a_func(p1, **kwds):\n    pass\n'
                       'a_func(p2=2, p1=1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual('def a_func(p1, **kwds):\n    pass\n'
                          'a_func(1, p2=2)\n', self.mod.read())

    def test_removing_arguments(self):
        self.mod.write('def a_func(p1):\n    pass\na_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(0)]))
        self.assertEqual('def a_func():\n    pass\na_func()\n',
                          self.mod.read())

    def test_removing_arguments_with_multiple_args(self):
        self.mod.write('def a_func(p1, p2):\n    pass\na_func(1, 2)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(0)]))
        self.assertEqual('def a_func(p2):\n    pass\na_func(2)\n',
                          self.mod.read())

    def test_removing_arguments_passed_as_keywords(self):
        self.mod.write('def a_func(p1):\n    pass\na_func(p1=1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(0)]))
        self.assertEqual('def a_func():\n    pass\na_func()\n',
                          self.mod.read())

    def test_removing_arguments_with_defaults(self):
        self.mod.write('def a_func(p1=1):\n    pass\na_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(0)]))
        self.assertEqual('def a_func():\n    pass\na_func()\n',
                          self.mod.read())

    def test_removing_arguments_star_args(self):
        self.mod.write('def a_func(p1, *args):\n    pass\na_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(1)]))
        self.assertEqual('def a_func(p1):\n    pass\na_func(1)\n',
                          self.mod.read())

    def test_removing_keyword_arg(self):
        self.mod.write('def a_func(p1, **kwds):\n    pass\na_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(1)]))
        self.assertEqual('def a_func(p1):\n    pass\na_func(1)\n',
                          self.mod.read())

    def test_removing_keyword_arg2(self):
        self.mod.write('def a_func(p1, *args, **kwds):\n    pass\na_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(2)]))
        self.assertEqual('def a_func(p1, *args):\n    pass\na_func(1)\n',
                          self.mod.read())

    # XXX: What to do here for star args?
    @unittest.skip("How to deal with start args?")
    def xxx_test_removing_arguments_star_args2(self):
        self.mod.write('def a_func(p1, *args):\n    pass\n'
                       'a_func(2, 3, p1=1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(1)]))
        self.assertEqual('def a_func(p1):\n    pass\na_func(p1=1)\n',
                          self.mod.read())

    # XXX: What to do here for star args?
    def xxx_test_removing_arguments_star_args3(self):
        self.mod.write('def a_func(p1, *args):\n    pass\n'
                       'a_func(*[1, 2, 3])\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(1)]))
        self.assertEqual('def a_func(p1):\n    pass\na_func(*[1, 2, 3])\n',
                          self.mod.read())

    def test_adding_arguments_for_normal_args_changing_definition(self):
        self.mod.write('def a_func():\n    pass\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentAdder(0, 'p1')]))
        self.assertEqual('def a_func(p1):\n    pass\n', self.mod.read())

    def test_adding_arguments_for_normal_args_with_defaults(self):
        self.mod.write('def a_func():\n    pass\na_func()\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        adder = change_signature.ArgumentAdder(0, 'p1', 'None')
        self.project.do(signature.get_changes([adder]))
        self.assertEqual('def a_func(p1=None):\n    pass\na_func()\n',
                          self.mod.read())

    def test_adding_arguments_for_normal_args_changing_calls(self):
        self.mod.write('def a_func():\n    pass\na_func()\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        adder = change_signature.ArgumentAdder(0, 'p1', 'None', '1')
        self.project.do(signature.get_changes([adder]))
        self.assertEqual('def a_func(p1=None):\n    pass\na_func(1)\n',
                          self.mod.read())

    def test_adding_arguments_for_norm_args_chang_calls_with_kwords(self):
        self.mod.write('def a_func(p1=0):\n    pass\na_func()\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        adder = change_signature.ArgumentAdder(1, 'p2', '0', '1')
        self.project.do(signature.get_changes([adder]))
        self.assertEqual('def a_func(p1=0, p2=0):\n    pass\na_func(p2=1)\n',
                          self.mod.read())

    def test_adding_arguments_for_norm_args_chang_calls_with_no_value(self):
        self.mod.write('def a_func(p2=0):\n    pass\na_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        adder = change_signature.ArgumentAdder(0, 'p1', '0', None)
        self.project.do(signature.get_changes([adder]))
        self.assertEqual('def a_func(p1=0, p2=0):\n    pass\na_func(p2=1)\n',
                          self.mod.read())

    def test_adding_duplicate_parameter_and_raising_exceptions(self):
        self.mod.write('def a_func(p1):\n    pass\n')
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            signature = change_signature.ChangeSignature(
                self.project, self.mod, self.mod.read().index('a_func') + 1)
            self.project.do(signature.get_changes(
                [change_signature.ArgumentAdder(1, 'p1')]))

    def test_inlining_default_arguments(self):
        self.mod.write('def a_func(p1=0):\n    pass\na_func()\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentDefaultInliner(0)]))
        self.assertEqual('def a_func(p1=0):\n    pass\n'
                          'a_func(0)\n', self.mod.read())

    def test_inlining_default_arguments2(self):
        self.mod.write('def a_func(p1=0):\n    pass\na_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentDefaultInliner(0)]))
        self.assertEqual('def a_func(p1=0):\n    pass\n'
                          'a_func(1)\n', self.mod.read())

    def test_preserving_args_and_keywords_order(self):
        self.mod.write('def a_func(*args, **kwds):\n    pass\n'
                       'a_func(3, 1, 2, a=1, c=3, b=2)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentNormalizer()]))
        self.assertEqual('def a_func(*args, **kwds):\n    pass\n'
                          'a_func(3, 1, 2, a=1, c=3, b=2)\n', self.mod.read())

    def test_change_order_for_only_one_parameter(self):
        self.mod.write('def a_func(p1):\n    pass\na_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentReorderer([0])]))
        self.assertEqual('def a_func(p1):\n    pass\na_func(1)\n',
                          self.mod.read())

    def test_change_order_for_two_parameter(self):
        self.mod.write('def a_func(p1, p2):\n    pass\na_func(1, 2)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentReorderer([1, 0])]))
        self.assertEqual('def a_func(p2, p1):\n    pass\na_func(2, 1)\n',
                          self.mod.read())

    def test_reordering_multi_line_function_headers(self):
        self.mod.write('def a_func(p1,\n p2):\n    pass\na_func(1, 2)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentReorderer([1, 0])]))
        self.assertEqual('def a_func(p2, p1):\n    pass\na_func(2, 1)\n',
                          self.mod.read())

    def test_changing_order_with_static_params(self):
        self.mod.write('def a_func(p1, p2=0, p3=0):\n    pass\na_func(1, 2)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentReorderer([0, 2, 1])]))
        self.assertEqual('def a_func(p1, p3=0, p2=0):\n    pass\n'
                          'a_func(1, p2=2)\n', self.mod.read())

    def test_doing_multiple_changes(self):
        changers = []
        self.mod.write('def a_func(p1):\n    pass\na_func(1)\n')
        changers.append(change_signature.ArgumentRemover(0))
        changers.append(change_signature.ArgumentAdder(0, 'p2', None, None))
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        signature.get_changes(changers).do()
        self.assertEqual('def a_func(p2):\n    pass\na_func()\n',
                          self.mod.read())

    def test_doing_multiple_changes2(self):
        changers = []
        self.mod.write('def a_func(p1, p2):\n    pass\na_func(p2=2)\n')
        changers.append(change_signature.ArgumentAdder(2, 'p3', None, '3'))
        changers.append(change_signature.ArgumentReorderer([1, 0, 2]))
        changers.append(change_signature.ArgumentRemover(1))
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        signature.get_changes(changers).do()
        self.assertEqual('def a_func(p2, p3):\n    pass\na_func(2, 3)\n',
                          self.mod.read())

    def test_changing_signature_in_subclasses(self):
        self.mod.write(
            'class A(object):\n    def a_method(self):\n        pass\n'
            'class B(A):\n    def a_method(self):\n        pass\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_method') + 1)
        signature.get_changes([change_signature.ArgumentAdder(1, 'p1')],
                              in_hierarchy=True).do()
        self.assertEqual(
            'class A(object):\n    def a_method(self, p1):\n        pass\n'
            'class B(A):\n    def a_method(self, p1):\n        pass\n',
            self.mod.read())

    def test_differentiating_class_accesses_from_instance_accesses(self):
        self.mod.write(
            'class A(object):\n    def a_func(self, param):\n        pass\n'
            'a_var = A()\nA.a_func(a_var, param=1)')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('a_func') + 1)
        self.project.do(signature.get_changes(
            [change_signature.ArgumentRemover(1)]))
        self.assertEqual(
            'class A(object):\n    def a_func(self):\n        pass\n'
            'a_var = A()\nA.a_func(a_var)', self.mod.read())

    def test_changing_signature_for_constructors(self):
        self.mod.write(
            'class C(object):\n    def __init__(self, p):\n        pass\n'
            'c = C(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('C') + 1)
        signature.get_changes([change_signature.ArgumentRemover(1)]).do()
        self.assertEqual(
            'class C(object):\n    def __init__(self):\n        pass\n'
            'c = C()\n',
            self.mod.read())

    def test_changing_signature_for_constructors2(self):
        self.mod.write(
            'class C(object):\n    def __init__(self, p):\n        pass\n'
            'c = C(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('__init__') + 1)
        signature.get_changes([change_signature.ArgumentRemover(1)]).do()
        self.assertEqual(
            'class C(object):\n    def __init__(self):\n        pass\n'
            'c = C()\n',
            self.mod.read())

    def test_changing_signature_for_constructors_when_using_super(self):
        self.mod.write(
            'class A(object):\n    def __init__(self, p):\n        pass\n'
            'class B(A):\n    '
            'def __init__(self, p):\n        super(B, self).__init__(p)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().index('__init__') + 1)
        signature.get_changes([change_signature.ArgumentRemover(1)]).do()
        self.assertEqual(
            'class A(object):\n    def __init__(self):\n        pass\n'
            'class B(A):\n    '
            'def __init__(self, p):\n        super(B, self).__init__()\n',
            self.mod.read())

    def test_redordering_arguments_reported_by_mft(self):
        self.mod.write('def f(a, b, c):\n    pass\nf(1, 2, 3)\n')
        signature = change_signature.ChangeSignature(
            self.project, self.mod, self.mod.read().rindex('f'))
        signature.get_changes(
            [change_signature.ArgumentReorderer([1, 2, 0])]).do()
        self.assertEqual('def f(b, c, a):\n    pass\nf(2, 3, 1)\n',
                          self.mod.read())

    def test_resources_parameter(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('def a_func(param):\n    pass\n')
        self.mod.write('import mod1\nmod1.a_func(1)\n')
        signature = change_signature.ChangeSignature(
            self.project, mod1, mod1.read().index('a_func') + 1)
        signature.get_changes([change_signature.ArgumentRemover(0)],
                              resources=[mod1]).do()
        self.assertEqual('import mod1\nmod1.a_func(1)\n', self.mod.read())
        self.assertEqual('def a_func():\n    pass\n', mod1.read())

    def test_reordering_and_automatic_defaults(self):
        code = 'def f(p1, p2=2):\n' \
               '    pass\n' \
               'f(1, 2)\n'
        self.mod.write(code)
        signature = change_signature.ChangeSignature(
            self.project, self.mod, code.index('f('))
        reorder = change_signature.ArgumentReorderer([1, 0], autodef='1')
        signature.get_changes([reorder]).do()
        expected = 'def f(p2=2, p1=1):\n' \
                   '    pass\n' \
                   'f(2, 1)\n'
        self.assertEqual(expected, self.mod.read())


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
import ast
from ast import *

from rope.base import fscommands

try:
    unicode
except NameError:
    unicode = str


def parse(source, filename='<string>'):
    # NOTE: the raw string should be given to `compile` function
    if isinstance(source, unicode):
        source = fscommands.unicode_to_file_data(source)
    if b'\r' in source:
        source = source.replace(b'\r\n', b'\n').replace(b'\r', b'\n')
    if not source.endswith(b'\n'):
        source += b'\n'
    try:
        return ast.parse(source, filename='<unknown>')
    except (TypeError, ValueError) as e:
        error = SyntaxError()
        error.lineno = 1
        error.filename = filename
        error.msg = str(e)
        raise error


def walk(node, walker):
    """Walk the syntax tree"""
    method_name = '_' + node.__class__.__name__
    method = getattr(walker, method_name, None)
    if method is not None:
        if isinstance(node, ast.ImportFrom) and node.module is None:
            # In python < 2.7 ``node.module == ''`` for relative imports
            # but for python 2.7 it is None. Generalizing it to ''.
            node.module = ''
        return method(node)
    for child in get_child_nodes(node):
        walk(child, walker)


def get_child_nodes(node):
    if isinstance(node, ast.Module):
        return node.body
    result = []
    if node._fields is not None:
        for name in node._fields:
            child = getattr(node, name)
            if isinstance(child, list):
                for entry in child:
                    if isinstance(entry, ast.AST):
                        result.append(entry)
            if isinstance(child, ast.AST):
                result.append(child)
    return result


def call_for_nodes(node, callback, recursive=False):
    """If callback returns `True` the child nodes are skipped"""
    result = callback(node)
    if recursive and not result:
        for child in get_child_nodes(node):
            call_for_nodes(child, callback, recursive)


def get_children(node):
    result = []
    if node._fields is not None:
        for name in node._fields:
            if name in ['lineno', 'col_offset']:
                continue
            child = getattr(node, name)
            result.append(child)
    return result



import base64
import hashlib
import hmac
import multiprocessing
try:
    import cPickle as pickle
except ImportError:
    import pickle
import socket
try:
    import unittest2 as unittest
except ImportError:
    import unittest


from rope.base.oi import doa


class DOATest(unittest.TestCase):

    def try_CVE_2014_3539_exploit(self, receiver, payload):
        # Simulated attacker writing to the socket
        def attacker(data_port):
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(('127.0.0.1', data_port))
            s_file = s.makefile('wb')
            s_file.write(payload)
            s.close()

        # Assume the attacker guesses the port correctly; 3037 is used by
        # default if it is available.
        attacker_proc = multiprocessing.Process(target=attacker,
                                                args=(receiver.data_port,))

        attacker_proc.start()
        received_objs = list(receiver.receive_data())
        attacker_proc.join()
        return received_objs
        
    def test_CVE_2014_3539_no_encoding(self):
        # Attacker sends pickled data to the receiver socket.
        receiver = doa._SocketReceiver()

        payload = pickle.dumps('def foo():\n    return 123\n')
        received_objs = self.try_CVE_2014_3539_exploit(receiver, payload)

        # Make sure the exploit did not run
        self.assertEqual(0, len(received_objs))
        
    def test_CVE_2014_3539_signature_mismatch(self):
        # Attacker sends well-formed data with an incorrect signature.
        receiver = doa._SocketReceiver()

        pickled_data = pickle.dumps('def foo():\n    return 123\n',
                                    pickle.HIGHEST_PROTOCOL)
        digest = hmac.new(b'invalid-key', pickled_data, hashlib.sha256).digest()
        payload = (base64.b64encode(digest) + b':' +
                   base64.b64encode(pickled_data) + b'\n')
        received_objs = self.try_CVE_2014_3539_exploit(receiver, payload)

        # Make sure the exploit did not run
        self.assertEqual(0, len(received_objs))
        
    def test_CVE_2014_3539_sanity(self):
        # Tests that sending valid, signed data on the socket does work.
        receiver = doa._SocketReceiver()

        pickled_data = base64.b64encode(
            pickle.dumps('def foo():\n    return 123\n',
                         pickle.HIGHEST_PROTOCOL))
        digest = hmac.new(receiver.key, pickled_data, hashlib.sha256).digest()
        payload = (base64.b64encode(digest) + b':' + pickled_data + b'\n')
        received_objs = self.try_CVE_2014_3539_exploit(receiver, payload)

        # Make sure the exploit did not run
        self.assertEqual(1, len(received_objs))

    def test_compare_digest_compat(self):
        self.assertTrue(doa._compat_compare_digest('', ''))
        self.assertTrue(doa._compat_compare_digest('abc', 'abc'))
        self.assertFalse(doa._compat_compare_digest('abc', 'abd'))
        self.assertFalse(doa._compat_compare_digest('abc', 'abcd'))


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(DOATest))
    return result

if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

import rope.base.exceptions
from rope.refactor import inline
from ropetest import testutils


class InlineTest(unittest.TestCase):

    def setUp(self):
        super(InlineTest, self).setUp()
        self.project = testutils.sample_project()
        self.pycore = self.project.pycore
        self.mod = testutils.create_module(self.project, 'mod')
        self.mod2 = testutils.create_module(self.project, 'mod2')

    def tearDown(self):
        testutils.remove_project(self.project)
        super(InlineTest, self).tearDown()

    def _inline(self, code, offset, **kwds):
        self.mod.write(code)
        self._inline2(self.mod, offset, **kwds)
        return self.mod.read()

    def _inline2(self, resource, offset, **kwds):
        inliner = inline.create_inline(self.project, resource, offset)
        changes = inliner.get_changes(**kwds)
        self.project.do(changes)
        return self.mod.read()

    def test_simple_case(self):
        code = 'a_var = 10\nanother_var = a_var\n'
        refactored = self._inline(code, code.index('a_var') + 1)
        self.assertEqual('another_var = 10\n', refactored)

    def test_empty_case(self):
        code = 'a_var = 10\n'
        refactored = self._inline(code, code.index('a_var') + 1)
        self.assertEqual('', refactored)

    def test_long_definition(self):
        code = 'a_var = 10 + (10 + 10)\nanother_var = a_var\n'
        refactored = self._inline(code, code.index('a_var') + 1)
        self.assertEqual('another_var = 10 + (10 + 10)\n', refactored)

    def test_explicit_continuation(self):
        code = 'a_var = (10 +\n 10)\nanother_var = a_var\n'
        refactored = self._inline(code, code.index('a_var') + 1)
        self.assertEqual('another_var = (10 + 10)\n', refactored)

    def test_implicit_continuation(self):
        code = 'a_var = 10 +\\\n       10\nanother_var = a_var\n'
        refactored = self._inline(code, code.index('a_var') + 1)
        self.assertEqual('another_var = 10 + 10\n', refactored)

    def test_inlining_at_the_end_of_input(self):
        code = 'a = 1\nb = a'
        refactored = self._inline(code, code.index('a') + 1)
        self.assertEqual('b = 1', refactored)

    def test_on_classes(self):
        code = 'class AClass(object):\n    pass\n'
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline(code, code.index('AClass') + 1)

    def test_multiple_assignments(self):
        code = 'a_var = 10\na_var = 20\n'
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline(code, code.index('a_var') + 1)

    def test_tuple_assignments(self):
        code = 'a_var, another_var = (20, 30)\n'
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline(code, code.index('a_var') + 1)

    def test_on_unknown_vars(self):
        code = 'a_var = another_var\n'
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline(code, code.index('another_var') + 1)

    def test_attribute_inlining(self):
        code = 'class A(object):\n    def __init__(self):\n' \
               '        self.an_attr = 3\n        range(self.an_attr)\n'
        refactored = self._inline(code, code.index('an_attr') + 1)
        expected = 'class A(object):\n    def __init__(self):\n' \
                   '        range(3)\n'
        self.assertEqual(expected, refactored)

    def test_attribute_inlining2(self):
        code = 'class A(object):\n    def __init__(self):\n' \
               '        self.an_attr = 3\n        range(self.an_attr)\n' \
               'a = A()\nrange(a.an_attr)'
        refactored = self._inline(code, code.index('an_attr') + 1)
        expected = 'class A(object):\n    def __init__(self):\n' \
                   '        range(3)\n' \
                   'a = A()\nrange(3)'
        self.assertEqual(expected, refactored)

    def test_a_function_with_no_occurance(self):
        self.mod.write('def a_func():\n    pass\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('', self.mod.read())

    def test_a_function_with_no_occurance2(self):
        self.mod.write('a_var = 10\ndef a_func():\n    pass\nprint(a_var)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('a_var = 10\nprint(a_var)\n', self.mod.read())

    def test_replacing_calls_with_function_definition_in_other_modules(self):
        self.mod.write('def a_func():\n    print(1)\n')
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('import mod\nmod.a_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('import mod\nprint(1)\n', mod1.read())

    def test_replacing_calls_with_function_definition_in_other_modules2(self):
        self.mod.write('def a_func():\n    print(1)\n')
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('import mod\nif True:\n    mod.a_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('import mod\nif True:\n    print(1)\n', mod1.read())

    def test_replacing_calls_with_method_definition_in_other_modules(self):
        self.mod.write('class A(object):\n    var = 10\n'
                       '    def a_func(self):\n        print(1)\n')
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('import mod\nmod.A().a_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('import mod\nprint(1)\n', mod1.read())
        self.assertEqual('class A(object):\n    var = 10\n', self.mod.read())

    def test_replacing_calls_with_function_definition_in_defining_module(self):
        self.mod.write('def a_func():\n    print(1)\na_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('print(1)\n', self.mod.read())

    def test_replac_calls_with_function_definition_in_defining_module2(self):
        self.mod.write('def a_func():\n    '
                       'for i in range(10):\n        print(1)\na_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('for i in range(10):\n    print(1)\n',
                          self.mod.read())

    def test_replacing_calls_with_method_definition_in_defining_modules(self):
        self.mod.write('class A(object):\n    var = 10\n'
                       '    def a_func(self):\n        print(1)\nA().a_func()')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('class A(object):\n    var = 10\nprint(1)\n',
                          self.mod.read())

    def test_parameters_with_the_same_name_as_passed(self):
        self.mod.write('def a_func(var):\n    '
                       'print(var)\nvar = 1\na_func(var)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('var = 1\nprint(var)\n', self.mod.read())

    def test_parameters_with_the_same_name_as_passed2(self):
        self.mod.write('def a_func(var):\n    '
                       'print(var)\nvar = 1\na_func(var=var)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('var = 1\nprint(var)\n', self.mod.read())

    def test_simple_parameters_renaming(self):
        self.mod.write('def a_func(param):\n    '
                       'print(param)\nvar = 1\na_func(var)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('var = 1\nprint(var)\n', self.mod.read())

    def test_simple_parameters_renaming_for_multiple_params(self):
        self.mod.write('def a_func(param1, param2):\n    p = param1 + param2\n'
                       'var1 = 1\nvar2 = 1\na_func(var1, var2)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('var1 = 1\nvar2 = 1\np = var1 + var2\n',
                          self.mod.read())

    def test_parameters_renaming_for_passed_constants(self):
        self.mod.write('def a_func(param):\n    print(param)\na_func(1)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('print(1)\n', self.mod.read())

    def test_parameters_renaming_for_passed_statements(self):
        self.mod.write('def a_func(param):\n    '
                       'print(param)\na_func((1 + 2) / 3)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('print((1 + 2) / 3)\n', self.mod.read())

    def test_simple_parameters_renam_for_multiple_params_using_keywords(self):
        self.mod.write('def a_func(param1, param2):\n    '
                       'p = param1 + param2\n'
                       'var1 = 1\nvar2 = 1\n'
                       'a_func(param2=var1, param1=var2)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('var1 = 1\nvar2 = 1\np = var2 + var1\n',
                          self.mod.read())

    def test_simple_params_renam_for_multi_params_using_mixed_keywords(self):
        self.mod.write('def a_func(param1, param2):\n    p = param1 + param2\n'
                       'var1 = 1\nvar2 = 1\na_func(var2, param2=var1)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('var1 = 1\nvar2 = 1\np = var2 + var1\n',
                          self.mod.read())

    def test_simple_putting_in_default_arguments(self):
        self.mod.write('def a_func(param=None):\n    print(param)\n'
                       'a_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('print(None)\n', self.mod.read())

    def test_overriding_default_arguments(self):
        self.mod.write('def a_func(param1=1, param2=2):'
                       '\n    print(param1, param2)\n'
                       'a_func(param2=3)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('print(1, 3)\n', self.mod.read())

    def test_arguments_containing_comparisons(self):
        self.mod.write('def a_func(param1, param2, param3):'
                       '\n    param2.name\n'
                       'a_func(2 <= 1, item, True)\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('item.name\n', self.mod.read())

    def test_badly_formatted_text(self):
        self.mod.write('def a_func  (  param1 =  1 ,param2 = 2 )  :'
                       '\n    print(param1, param2)\n'
                       'a_func  ( param2 \n  = 3 )  \n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('print(1, 3)\n', self.mod.read())

    def test_passing_first_arguments_for_methods(self):
        a_class = 'class A(object):\n' \
                  '    def __init__(self):\n' \
                  '        self.var = 1\n' \
                  '        self.a_func(self.var)\n' \
                  '    def a_func(self, param):\n' \
                  '        print(param)\n'
        self.mod.write(a_class)
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        expected = 'class A(object):\n' \
                   '    def __init__(self):\n' \
                   '        self.var = 1\n' \
                   '        print(self.var)\n'
        self.assertEqual(expected, self.mod.read())

    def test_passing_first_arguments_for_methods2(self):
        a_class = 'class A(object):\n' \
                  '    def __init__(self):\n' \
                  '        self.var = 1\n' \
                  '    def a_func(self, param):\n' \
                  '        print(param, self.var)\n' \
                  'an_a = A()\n' \
                  'an_a.a_func(1)\n'
        self.mod.write(a_class)
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        expected = 'class A(object):\n' \
                   '    def __init__(self):\n' \
                   '        self.var = 1\n' \
                   'an_a = A()\n' \
                   'print(1, an_a.var)\n'
        self.assertEqual(expected, self.mod.read())

    def test_passing_first_arguments_for_methods3(self):
        a_class = 'class A(object):\n' \
                  '    def __init__(self):\n' \
                  '        self.var = 1\n' \
                  '    def a_func(self, param):\n' \
                  '        print(param, self.var)\n' \
                  'an_a = A()\n' \
                  'A.a_func(an_a, 1)\n'
        self.mod.write(a_class)
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        expected = 'class A(object):\n' \
                   '    def __init__(self):\n' \
                   '        self.var = 1\n' \
                   'an_a = A()\n' \
                   'print(1, an_a.var)\n'
        self.assertEqual(expected, self.mod.read())

    def test_inlining_staticmethods(self):
        a_class = 'class A(object):\n' \
                  '    @staticmethod\n' \
                  '    def a_func(param):\n' \
                  '        print(param)\n' \
                  'A.a_func(1)\n'
        self.mod.write(a_class)
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        expected = 'class A(object):\n' \
            '    pass\n' \
            'print(1)\n'
        self.assertEqual(expected, self.mod.read())

    def test_static_methods2(self):
        a_class = 'class A(object):\n' \
                  '    var = 10\n' \
                  '    @staticmethod\n' \
                  '    def a_func(param):\n' \
                  '        print(param)\n' \
                  'an_a = A()\n' \
                  'an_a.a_func(1)\n' \
                  'A.a_func(2)\n'
        self.mod.write(a_class)
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        expected = 'class A(object):\n' \
            '    var = 10\n' \
            'an_a = A()\n' \
            'print(1)\n' \
            'print(2)\n'
        self.assertEqual(expected, self.mod.read())

    def test_inlining_classmethods(self):
        a_class = 'class A(object):\n' \
                  '    @classmethod\n' \
                  '    def a_func(cls, param):\n' \
                  '        print(param)\n' \
                  'A.a_func(1)\n'
        self.mod.write(a_class)
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        expected = 'class A(object):\n' \
                   '    pass\n' \
                   'print(1)\n'
        self.assertEqual(expected, self.mod.read())

    def test_inlining_classmethods2(self):
        a_class = 'class A(object):\n' \
                  '    @classmethod\n' \
                  '    def a_func(cls, param):\n' \
                  '        return cls\n' \
                  'print(A.a_func(1))\n'
        self.mod.write(a_class)
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        expected = 'class A(object):\n' \
                   '    pass\n' \
                   'print(A)\n'
        self.assertEqual(expected, self.mod.read())

    def test_simple_return_values_and_inlining_functions(self):
        self.mod.write('def a_func():\n    return 1\na = a_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('a = 1\n',
                          self.mod.read())

    def test_simple_return_values_and_inlining_lonely_functions(self):
        self.mod.write('def a_func():\n    return 1\na_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('1\n', self.mod.read())

    def test_empty_returns_and_inlining_lonely_functions(self):
        self.mod.write('def a_func():\n    '
                       'if True:\n        return\na_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('if True:\n    pass\n', self.mod.read())

    def test_multiple_returns(self):
        self.mod.write('def less_than_five(var):\n    if var < 5:\n'
                       '        return True\n    return False\n'
                       'a = less_than_five(2)\n')
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline2(self.mod, self.mod.read().index('less') + 1)

    def test_multiple_returns_and_not_using_the_value(self):
        self.mod.write('def less_than_five(var):\n    if var < 5:\n'
                       '        return True\n    '
                       'return False\nless_than_five(2)\n')
        self._inline2(self.mod, self.mod.read().index('less') + 1)
        self.assertEqual('if 2 < 5:\n    True\nFalse\n', self.mod.read())

    def test_raising_exception_for_list_arguments(self):
        self.mod.write('def a_func(*args):\n    print(args)\na_func(1)\n')
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline2(self.mod, self.mod.read().index('a_func') + 1)

    def test_raising_exception_for_list_keywods(self):
        self.mod.write('def a_func(**kwds):\n    print(kwds)\na_func(n=1)\n')
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline2(self.mod, self.mod.read().index('a_func') + 1)

    def test_function_parameters_and_returns_in_other_functions(self):
        code = 'def a_func(param1, param2):\n' \
               '    return param1 + param2\n' \
               'range(a_func(20, param2=abs(10)))\n'
        self.mod.write(code)
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('range(20 + abs(10))\n', self.mod.read())

    def test_function_references_other_than_call(self):
        self.mod.write('def a_func(param):\n    print(param)\nf = a_func\n')
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline2(self.mod, self.mod.read().index('a_func') + 1)

    def test_function_referencing_itself(self):
        self.mod.write('def a_func(var):\n    func = a_func\n')
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline2(self.mod, self.mod.read().index('a_func') + 1)

    def test_recursive_functions(self):
        self.mod.write('def a_func(var):\n    a_func(var)\n')
        with self.assertRaises(rope.base.exceptions.RefactoringError):
            self._inline2(self.mod, self.mod.read().index('a_func') + 1)

    # TODO: inlining on function parameters
    def xxx_test_inlining_function_default_parameters(self):
        self.mod.write('def a_func(p1=1):\n    pass\na_func()\n')
        self._inline2(self.mod, self.mod.read().index('p1') + 1)
        self.assertEqual('def a_func(p1=1):\n    pass\na_func()\n',
                          self.mod.read())

    def test_simple_inlining_after_extra_indented_lines(self):
        self.mod.write('def a_func():\n    for i in range(10):\n        pass\n'
                       'if True:\n    pass\na_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('if True:\n    pass\nfor i in range(10):'
                          '\n    pass\n',
                          self.mod.read())

    def test_inlining_a_function_with_pydoc(self):
        self.mod.write('def a_func():\n    """docs"""\n    a = 1\na_func()')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('a = 1\n', self.mod.read())

    def test_inlining_methods(self):
        self.mod.write("class A(object):\n    name = 'hey'\n"
                       "    def get_name(self):\n        return self.name\n"
                       "a = A()\nname = a.get_name()\n")
        self._inline2(self.mod, self.mod.read().rindex('get_name') + 1)
        self.assertEqual("class A(object):\n    name = 'hey'\n"
                          "a = A()\nname = a.name\n", self.mod.read())

    def test_simple_returns_with_backslashes(self):
        self.mod.write('def a_func():\n    return 1'
                       '\\\n        + 2\na = a_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('a = 1 + 2\n', self.mod.read())

    def test_a_function_with_pass_body(self):
        self.mod.write('def a_func():\n    print(1)\na = a_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func') + 1)
        self.assertEqual('print(1)\na = None\n', self.mod.read())

    def test_inlining_the_last_method_of_a_class(self):
        self.mod.write('class A(object):\n'
                       '    def a_func(self):\n        pass\n')
        self._inline2(self.mod, self.mod.read().rindex('a_func') + 1)
        self.assertEqual('class A(object):\n    pass\n',
                          self.mod.read())

    def test_adding_needed_imports_in_the_dest_module(self):
        self.mod.write('import sys\n\ndef ver():\n    print(sys.version)\n')
        self.mod2.write('import mod\n\nmod.ver()')
        self._inline2(self.mod, self.mod.read().index('ver') + 1)
        self.assertEqual('import mod\nimport sys\n\nprint(sys.version)\n',
                          self.mod2.read())

    def test_adding_needed_imports_in_the_dest_module_removing_selfs(self):
        self.mod.write('import mod2\n\ndef f():\n    print(mod2.var)\n')
        self.mod2.write('import mod\n\nvar = 1\nmod.f()\n')
        self._inline2(self.mod, self.mod.read().index('f(') + 1)
        self.assertEqual('import mod\n\nvar = 1\nprint(var)\n',
                          self.mod2.read())

    def test_handling_relative_imports_when_inlining(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod3 = testutils.create_module(self.project, 'mod3', pkg)
        mod4 = testutils.create_module(self.project, 'mod4', pkg)
        mod4.write('var = 1\n')
        mod3.write('from . import mod4\n\ndef f():\n    print(mod4.var)\n')
        self.mod.write('import pkg.mod3\n\npkg.mod3.f()\n')
        self._inline2(self.mod, self.mod.read().index('f(') + 1)
        # Cannot determine the exact import
        self.assertTrue('\n\nprint(mod4.var)\n' in self.mod.read())

    def test_adding_needed_imports_for_elements_in_source(self):
        self.mod.write('def f1():\n    return f2()\ndef f2():\n    return 1\n')
        self.mod2.write('import mod\n\nprint(mod.f1())\n')
        self._inline2(self.mod, self.mod.read().index('f1') + 1)
        self.assertEqual('import mod\nfrom mod import f2\n\nprint(f2())\n',
                          self.mod2.read())

    def test_relative_imports_and_changing_inlining_body(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod3 = testutils.create_module(self.project, 'mod3', pkg)
        mod4 = testutils.create_module(self.project, 'mod4', pkg)
        mod4.write('var = 1\n')
        mod3.write('import mod4\n\ndef f():\n    print(mod4.var)\n')
        self.mod.write('import pkg.mod3\n\npkg.mod3.f()\n')
        self._inline2(self.mod, self.mod.read().index('f(') + 1)
        self.assertEqual(
            'import pkg.mod3\nimport pkg.mod4\n\nprint(pkg.mod4.var)\n',
            self.mod.read())

    def test_inlining_with_different_returns(self):
        self.mod.write('def f(p):\n    return p\n'
                       'print(f(1))\nprint(f(2))\nprint(f(1))\n')
        self._inline2(self.mod, self.mod.read().index('f(') + 1)
        self.assertEqual('print(1)\nprint(2)\nprint(1)\n',
                          self.mod.read())

    def test_not_removing_definition_for_variables(self):
        code = 'a_var = 10\nanother_var = a_var\n'
        refactored = self._inline(code, code.index('a_var') + 1,
                                  remove=False)
        self.assertEqual('a_var = 10\nanother_var = 10\n', refactored)

    def test_not_removing_definition_for_methods(self):
        code = 'def func():\n    print(1)\n\nfunc()\n'
        refactored = self._inline(code, code.index('func') + 1,
                                  remove=False)
        self.assertEqual('def func():\n    print(1)\n\nprint(1)\n',
                          refactored)

    def test_only_current_for_methods(self):
        code = 'def func():\n    print(1)\n\nfunc()\nfunc()\n'
        refactored = self._inline(code, code.rindex('func') + 1,
                                  remove=False, only_current=True)
        self.assertEqual('def func():\n    print(1)\n\nfunc()\nprint(1)\n',
                          refactored)

    def test_only_current_for_variables(self):
        code = 'one = 1\n\na = one\nb = one\n'
        refactored = self._inline(code, code.rindex('one') + 1,
                                  remove=False, only_current=True)
        self.assertEqual('one = 1\n\na = one\nb = 1\n', refactored)

    def test_inlining_one_line_functions(self):
        code = 'def f(): return 1\nvar = f()\n'
        refactored = self._inline(code, code.rindex('f'))
        self.assertEqual('var = 1\n', refactored)

    def test_inlining_one_line_functions_with_breaks(self):
        code = 'def f(\np): return p\nvar = f(1)\n'
        refactored = self._inline(code, code.rindex('f'))
        self.assertEqual('var = 1\n', refactored)

    def test_inlining_one_line_functions_with_breaks2(self):
        code = 'def f(\n): return 1\nvar = f()\n'
        refactored = self._inline(code, code.rindex('f'))
        self.assertEqual('var = 1\n', refactored)

    def test_resources_parameter(self):
        self.mod.write('def a_func():\n    print(1)\n')
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('import mod\nmod.a_func()\n')
        self._inline2(self.mod, self.mod.read().index('a_func'),
                      resources=[self.mod])
        self.assertEqual('', self.mod.read())
        self.assertEqual('import mod\nmod.a_func()\n', mod1.read())

    def test_inlining_parameters(self):
        code = 'def f(p=1):\n    pass\nf()\n'
        result = self._inline(code, code.index('p'))
        self.assertEqual('def f(p=1):\n    pass\nf(1)\n', result)

    def test_inlining_function_with_line_breaks_in_args(self):
        code = 'def f(p): return p\nvar = f(1 +\n1)\n'
        refactored = self._inline(code, code.rindex('f'))
        self.assertEqual('var = 1 + 1\n', refactored)

    def test_inlining_variables_before_comparison(self):
        code = 'start = 1\nprint(start <= 2)\n'
        refactored = self._inline(code, code.index('start'))
        self.assertEqual('print(1 <= 2)\n', refactored)

    def test_inlining_variables_in_other_modules(self):
        self.mod.write('myvar = 1\n')
        self.mod2.write('import mod\nprint(mod.myvar)\n')
        self._inline2(self.mod, 2)
        self.assertEqual('import mod\nprint(1)\n', self.mod2.read())

    def test_inlining_variables_and_back_importing(self):
        self.mod.write('mainvar = 1\nmyvar = mainvar\n')
        self.mod2.write('import mod\nprint(mod.myvar)\n')
        self._inline2(self.mod, self.mod.read().index('myvar'))
        expected = 'import mod\n' \
                   'from mod import mainvar\n' \
                   'print(mainvar)\n'
        self.assertEqual(expected, self.mod2.read())

    def test_inlining_variables_and_importing_used_imports(self):
        self.mod.write('import sys\nmyvar = sys.argv\n')
        self.mod2.write('import mod\nprint(mod.myvar)\n')
        self._inline2(self.mod, self.mod.read().index('myvar'))
        expected = 'import mod\n' \
                   'import sys\n' \
                   'print(sys.argv)\n'
        self.assertEqual(expected, self.mod2.read())

    def test_inlining_variables_and_removing_old_froms(self):
        self.mod.write('var = 1\n')
        self.mod2.write('from mod import var\nprint(var)\n')
        self._inline2(self.mod2, self.mod2.read().rindex('var'))
        self.assertEqual('print(1)\n', self.mod2.read())

    def test_inlining_method_and_removing_old_froms(self):
        self.mod.write('def f():    return 1\n')
        self.mod2.write('from mod import f\nprint(f())\n')
        self._inline2(self.mod2, self.mod2.read().rindex('f'))
        self.assertEqual('print(1)\n', self.mod2.read())

    def test_inlining_functions_in_other_modules_and_only_current(self):
        code1 = 'def f():\n' \
                '    return 1\n' \
                'print(f())\n'
        code2 = 'import mod\n' \
                'print(mod.f())\n' \
                'print(mod.f())\n'
        self.mod.write(code1)
        self.mod2.write(code2)
        self._inline2(self.mod2, self.mod2.read().rindex('f'),
                      remove=False, only_current=True)
        expected2 = 'import mod\n' \
                    'print(mod.f())\n' \
                    'print(1)\n'
        self.assertEqual(code1, self.mod.read())
        self.assertEqual(expected2, self.mod2.read())

    def test_inlining_variables_in_other_modules_and_only_current(self):
        code1 = 'var = 1\n' \
                'print(var)\n'
        code2 = 'import mod\n' \
                'print(mod.var)\n' \
                'print(mod.var)\n'
        self.mod.write(code1)
        self.mod2.write(code2)
        self._inline2(self.mod2, self.mod2.read().rindex('var'),
                      remove=False, only_current=True)
        expected2 = 'import mod\n' \
                    'print(mod.var)\n' \
                    'print(1)\n'
        self.assertEqual(code1, self.mod.read())
        self.assertEqual(expected2, self.mod2.read())

    def test_inlining_does_not_change_string_constants(self):
        code = 'var = 1\n' \
               'print("var\\\n' \
               '")\n'
        expected = 'var = 1\n' \
                   'print("var\\\n' \
                   '")\n'
        refactored = self._inline(code, code.rindex('var'),
                                  remove=False, only_current=True, docs=False)
        self.assertEqual(expected, refactored)

    def test_inlining_does_change_string_constants_if_docs_is_set(self):
        code = 'var = 1\n' \
               'print("var\\\n' \
               '")\n'
        expected = 'var = 1\n' \
                   'print("1\\\n' \
                   '")\n'
        refactored = self._inline(code, code.rindex('var'),
                                  remove=False, only_current=True, docs=True)
        self.assertEqual(expected, refactored)


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(InlineTest))
    return result


if __name__ == '__main__':
    unittest.main()



import os.path
import shutil
import sys
import logging
logging.basicConfig(format='%(levelname)s:%(funcName)s:%(message)s',
                    level=logging.INFO)
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import rope.base.project
from rope.contrib import generate


def sample_project(root=None, foldername=None, **kwds):
    if root is None:
        root = 'sample_project'
        if foldername:
            root = foldername
        # HACK: Using ``/dev/shm/`` for faster tests
        if os.name == 'posix':
            if os.path.isdir('/dev/shm') and os.access('/dev/shm', os.W_OK):
                root = '/dev/shm/' + root
            elif os.path.isdir('/tmp') and os.access('/tmp', os.W_OK):
                root = '/tmp/' + root
    logging.debug("Using %s as root of the project.", root)
    # Using these prefs for faster tests
    prefs = {'save_objectdb': False, 'save_history': False,
             'validate_objectdb': False, 'automatic_soa': False,
             'ignored_resources': ['.ropeproject', '*.pyc'],
             'import_dynload_stdmods': False}
    prefs.update(kwds)
    remove_recursively(root)
    project = rope.base.project.Project(root, **prefs)
    return project

create_module = generate.create_module
create_package = generate.create_package


def remove_project(project):
    project.close()
    remove_recursively(project.address)


def remove_recursively(path):
    import time
    # windows sometimes raises exceptions instead of removing files
    if os.name == 'nt' or sys.platform == 'cygwin':
        for i in range(12):
            try:
                _remove_recursively(path)
            except OSError as e:
                if e.errno not in (13, 16, 32):
                    raise
                time.sleep(0.3)
            else:
                break
    else:
        _remove_recursively(path)


def _remove_recursively(path):
    if not os.path.exists(path):
        return
    if os.path.isfile(path):
        os.remove(path)
    else:
        shutil.rmtree(path)


def only_for(version):
    """Should be used as a decorator for a unittest.TestCase test method"""
    return unittest.skipIf(
        sys.version < version,
        'This test requires at least {0} version of Python.'.format(version))


def only_for_versions_lower(version):
    """Should be used as a decorator for a unittest.TestCase test method"""
    return unittest.skipIf(
        sys.version > version,
        'This test requires version of Python lower than {0}'.format(version))

def only_for_versions_higher(version):
    """Should be used as a decorator for a unittest.TestCase test method"""
    return unittest.skipIf(
        sys.version < version,
        'This test requires version of Python higher than {0}'.format(version))

def skipNotPOSIX():
    return unittest.skipIf(os.name != 'posix',
                           'This test works only on POSIX')


def time_limit(timeout):
    if not any(procname in sys.argv[0] for procname in {'pytest', 'py.test'}):
        # no-op when running tests without pytest
        return lambda *args, **kwargs: lambda func: func

    # do a local import so we don't import pytest when running without pytest
    import pytest

    # this prevents infinite loop/recursion from taking forever in CI
    return pytest.mark.time_limit(timeout)



import rope.base.codeanalyze
import rope.base.evaluate
import rope.base.pyobjects
from rope.base import taskhandle, exceptions, worder
from rope.contrib import fixsyntax
from rope.refactor import occurrences


def find_occurrences(project, resource, offset, unsure=False, resources=None,
                     in_hierarchy=False,
                     task_handle=taskhandle.NullTaskHandle()):
    """Return a list of `Location`

    If `unsure` is `True`, possible matches are returned, too.  You
    can use `Location.unsure` to see which are unsure occurrences.
    `resources` can be a list of `rope.base.resource.File` that
    should be searched for occurrences; if `None` all python files
    in the project are searched.

    """
    name = worder.get_name_at(resource, offset)
    this_pymodule = project.get_pymodule(resource)
    primary, pyname = rope.base.evaluate.eval_location2(
        this_pymodule, offset)

    def is_match(occurrence):
        return unsure
    finder = occurrences.create_finder(
        project, name, pyname, unsure=is_match,
        in_hierarchy=in_hierarchy, instance=primary)
    if resources is None:
        resources = project.get_python_files()
    job_set = task_handle.create_jobset('Finding Occurrences',
                                        count=len(resources))
    return _find_locations(finder, resources, job_set)


def find_implementations(project, resource, offset, resources=None,
                         task_handle=taskhandle.NullTaskHandle()):
    """Find the places a given method is overridden.

    Finds the places a method is implemented.  Returns a list of
    `Location`.
    """
    name = worder.get_name_at(resource, offset)
    this_pymodule = project.get_pymodule(resource)
    pyname = rope.base.evaluate.eval_location(this_pymodule, offset)
    if pyname is not None:
        pyobject = pyname.get_object()
        if not isinstance(pyobject, rope.base.pyobjects.PyFunction) or \
           pyobject.get_kind() != 'method':
            raise exceptions.BadIdentifierError('Not a method!')
    else:
        raise exceptions.BadIdentifierError('Cannot resolve the identifier!')

    def is_defined(occurrence):
        if not occurrence.is_defined():
            return False

    def not_self(occurrence):
        if occurrence.get_pyname().get_object() == pyname.get_object():
            return False
    filters = [is_defined, not_self,
               occurrences.InHierarchyFilter(pyname, True)]
    finder = occurrences.Finder(project, name, filters=filters)
    if resources is None:
        resources = project.get_python_files()
    job_set = task_handle.create_jobset('Finding Implementations',
                                        count=len(resources))
    return _find_locations(finder, resources, job_set)


def find_definition(project, code, offset, resource=None, maxfixes=1):
    """Return the definition location of the python name at `offset`

    A `Location` object is returned if the definition location can be
    determined, otherwise ``None`` is returned.
    """
    fixer = fixsyntax.FixSyntax(project, code, resource, maxfixes)
    pyname = fixer.pyname_at(offset)
    if pyname is not None:
        module, lineno = pyname.get_definition_location()
        name = rope.base.worder.Worder(code).get_word_at(offset)
        if lineno is not None:
            start = module.lines.get_line_start(lineno)

            def check_offset(occurrence):
                if occurrence.offset < start:
                    return False
            pyname_filter = occurrences.PyNameFilter(pyname)
            finder = occurrences.Finder(project, name,
                                        [check_offset, pyname_filter])
            for occurrence in finder.find_occurrences(pymodule=module):
                return Location(occurrence)


class Location(object):

    def __init__(self, occurrence):
        self.resource = occurrence.resource
        self.region = occurrence.get_word_range()
        self.offset = self.region[0]
        self.unsure = occurrence.is_unsure()
        self.lineno = occurrence.lineno


def _find_locations(finder, resources, job_set):
    result = []
    for resource in resources:
        job_set.started_job(resource.path)
        for occurrence in finder.find_occurrences(resource):
            result.append(Location(occurrence))
        job_set.finished_job()
    return result



try:
    import unittest2 as unittest
except ImportError:
    import unittest


import rope.base.history
import rope.contrib.changestack
import rope.base.change
from ropetest import testutils


class ChangeStackTest(unittest.TestCase):

    def setUp(self):
        super(ChangeStackTest, self).setUp()
        self.project = testutils.sample_project()

    def tearDown(self):
        testutils.remove_project(self.project)
        super(ChangeStackTest, self).tearDown()

    def test_change_stack(self):
        myfile = self.project.root.create_file('myfile.txt')
        myfile.write('1')
        stack = rope.contrib.changestack.ChangeStack(self.project)
        stack.push(rope.base.change.ChangeContents(myfile, '2'))
        self.assertEqual('2', myfile.read())
        stack.push(rope.base.change.ChangeContents(myfile, '3'))
        self.assertEqual('3', myfile.read())
        stack.pop_all()
        self.assertEqual('1', myfile.read())
        changes = stack.merged()
        self.project.do(changes)
        self.assertEqual('3', myfile.read())


if __name__ == '__main__':
    unittest.main()



import sys

from rope.base.builtins import File, BuiltinClass

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from rope.base import exceptions
from rope.base import libutils
from rope.base.pycore import _TextChangeDetector
from rope.base.pyobjects import get_base_type, AbstractFunction
from rope.base.pynamesdef import AssignedName
from ropetest import testutils


class PyCoreTest(unittest.TestCase):

    def setUp(self):
        super(PyCoreTest, self).setUp()
        self.project = testutils.sample_project()
        self.pycore = self.project.pycore

    def tearDown(self):
        testutils.remove_project(self.project)
        super(PyCoreTest, self).tearDown()

    def test_simple_module(self):
        testutils.create_module(self.project, 'mod')
        result = self.project.get_module('mod')
        self.assertEqual(get_base_type('Module'), result.type)
        self.assertEqual(0, len(result.get_attributes()))

    def test_nested_modules(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod = testutils.create_module(self.project, 'mod', pkg)  # noqa
        package = self.project.get_module('pkg')
        self.assertEqual(get_base_type('Module'), package.get_type())
        self.assertEqual(1, len(package.get_attributes()))
        module = package['mod'].get_object()
        self.assertEqual(get_base_type('Module'), module.get_type())

    def test_package(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod = testutils.create_module(self.project, 'mod', pkg)  # noqa
        result = self.project.get_module('pkg')
        self.assertEqual(get_base_type('Module'), result.type)

    def test_simple_class(self):
        mod = testutils.create_module(self.project, 'mod')
        mod.write('class SampleClass(object):\n    pass\n')
        mod_element = self.project.get_module('mod')
        result = mod_element['SampleClass'].get_object()
        self.assertEqual(get_base_type('Type'), result.get_type())

    def test_simple_function(self):
        mod = testutils.create_module(self.project, 'mod')
        mod.write('def sample_function():\n    pass\n')
        mod_element = self.project.get_module('mod')
        result = mod_element['sample_function'].get_object()
        self.assertEqual(get_base_type('Function'), result.get_type())

    def test_class_methods(self):
        mod = testutils.create_module(self.project, 'mod')
        code = 'class SampleClass(object):\n' \
               '    def sample_method(self):\n' \
               '        pass\n'
        mod.write(code)
        mod_element = self.project.get_module('mod')
        sample_class = mod_element['SampleClass'].get_object()
        self.assertTrue('sample_method' in sample_class)
        method = sample_class['sample_method'].get_object()
        self.assertEqual(get_base_type('Function'), method.get_type())

    def test_global_variable_without_type_annotation(self):
        mod = testutils.create_module(self.project, 'mod')
        mod.write('var = 10')
        mod_element = self.project.get_module('mod')
        var = mod_element['var']
        self.assertEqual(AssignedName, type(var))

    @testutils.only_for_versions_higher('3.6')
    def test_global_variable_with_type_annotation(self):
        mod = testutils.create_module(self.project, 'mod')
        mod.write('py3_var: str = foo_bar')
        mod_element = self.project.get_module('mod')
        py3_var = mod_element['py3_var']
        self.assertEqual(AssignedName, type(py3_var))

    def test_class_variables(self):
        mod = testutils.create_module(self.project, 'mod')
        mod.write('class SampleClass(object):\n    var = 10\n')
        mod_element = self.project.get_module('mod')
        sample_class = mod_element['SampleClass'].get_object()
        var = sample_class['var']  # noqa

    def test_class_attributes_set_in_init(self):
        mod = testutils.create_module(self.project, 'mod')
        mod.write('class C(object):\n'
                  '    def __init__(self):\n        self.var = 20\n')
        mod_element = self.project.get_module('mod')
        sample_class = mod_element['C'].get_object()
        var = sample_class['var']  # noqa

    def test_class_attributes_set_in_init_overwriting_a_defined(self):
        mod = testutils.create_module(self.project, 'mod')
        code = 'class C(object):\n' \
               '    def __init__(self):\n' \
               '        self.f = 20\n' \
               '    def f():\n' \
               '        pass\n'
        mod.write(code)
        mod_element = self.project.get_module('mod')
        sample_class = mod_element['C'].get_object()
        f = sample_class['f'].get_object()
        self.assertTrue(isinstance(f, AbstractFunction))

    def test_classes_inside_other_classes(self):
        mod = testutils.create_module(self.project, 'mod')
        code = 'class SampleClass(object):\n' \
               '    class InnerClass(object):\n' \
               '        pass\n\n'
        mod.write(code)
        mod_element = self.project.get_module('mod')
        sample_class = mod_element['SampleClass'].get_object()
        var = sample_class['InnerClass'].get_object()
        self.assertEqual(get_base_type('Type'), var.get_type())

    def test_non_existent_module(self):
        with self.assertRaises(exceptions.ModuleNotFoundError):
            self.project.get_module('doesnotexistmodule')

    def test_imported_names(self):
        testutils.create_module(self.project, 'mod1')
        mod = testutils.create_module(self.project, 'mod2')
        mod.write('import mod1\n')
        module = self.project.get_module('mod2')
        imported_sys = module['mod1'].get_object()
        self.assertEqual(get_base_type('Module'), imported_sys.get_type())

    def test_imported_as_names(self):
        testutils.create_module(self.project, 'mod1')
        mod = testutils.create_module(self.project, 'mod2')
        mod.write('import mod1 as my_import\n')
        module = self.project.get_module('mod2')
        imported_mod = module['my_import'].get_object()
        self.assertEqual(get_base_type('Module'), imported_mod.get_type())

    def test_get_string_module(self):
        mod = libutils.get_string_module(
            self.project, 'class Sample(object):\n    pass\n')
        sample_class = mod['Sample'].get_object()
        self.assertEqual(get_base_type('Type'), sample_class.get_type())

    def test_get_string_module_with_extra_spaces(self):
        mod = libutils.get_string_module(
            self.project, 'a = 10\n    ')  # noqa

    def test_parameter_info_for_functions(self):
        code = 'def func(param1, param2=10, *param3, **param4):\n    pass'
        mod = libutils.get_string_module(self.project, code)
        sample_function = mod['func']
        self.assertEqual(['param1', 'param2', 'param3', 'param4'],
                          sample_function.get_object().get_param_names())

    # FIXME: Not found modules
    def xxx_test_not_found_module_is_module(self):
        mod = libutils.get_string_module(
            self.project, 'import doesnotexist\n')
        self.assertEqual(get_base_type('Module'),
                          mod['doesnotexist'].
                          get_object().get_type())

    def test_mixing_scopes_and_objects_hierarchy(self):
        mod = libutils.get_string_module(self.project, 'var = 200\n')
        scope = mod.get_scope()
        self.assertTrue('var' in scope.get_names())

    def test_inheriting_base_class_attributes(self):
        code = 'class Base(object):\n' \
               '    def method(self):\n' \
               '        pass\n' \
               'class Derived(Base):\n' \
               '    pass\n'
        mod = libutils.get_string_module(self.project, code)
        derived = mod['Derived'].get_object()
        self.assertTrue('method' in derived)
        self.assertEqual(get_base_type('Function'),
                          derived['method'].get_object().get_type())

    def test_inheriting_multiple_base_class_attributes(self):
        code = 'class Base1(object):\n    def method1(self):\n        pass\n' \
               'class Base2(object):\n    def method2(self):\n        pass\n' \
               'class Derived(Base1, Base2):\n    pass\n'
        mod = libutils.get_string_module(self.project, code)
        derived = mod['Derived'].get_object()
        self.assertTrue('method1' in derived)
        self.assertTrue('method2' in derived)

    def test_inherit_multiple_base_class_attrs_with_the_same_name(self):
        code = 'class Base1(object):\n    def method(self):\n        pass\n' \
               'class Base2(object):\n    def method(self):\n        pass\n' \
               'class Derived(Base1, Base2):\n    pass\n'
        mod = libutils.get_string_module(self.project, code)
        base1 = mod['Base1'].get_object()
        derived = mod['Derived'].get_object()
        self.assertEqual(base1['method'].get_object(),
                          derived['method'].get_object())

    def test_inheriting_unknown_base_class(self):
        code = 'class Derived(NotFound):\n' \
               '    def f(self):\n' \
               '        pass\n'
        mod = libutils.get_string_module(self.project, code)
        derived = mod['Derived'].get_object()
        self.assertTrue('f' in derived)

    def test_module_creation(self):
        new_module = testutils.create_module(self.project, 'module')
        self.assertFalse(new_module.is_folder())
        self.assertEqual(self.project.get_resource('module.py'), new_module)

    def test_packaged_module_creation(self):
        package = self.project.root.create_folder('package')  # noqa
        new_module = testutils.create_module(self.project, 'package.module')
        self.assertEqual(self.project.get_resource('package/module.py'),
                          new_module)

    def test_packaged_module_creation_with_nested_src(self):
        src = self.project.root.create_folder('src')
        src.create_folder('pkg')
        new_module = testutils.create_module(self.project, 'pkg.mod', src)
        self.assertEqual(self.project.get_resource('src/pkg/mod.py'),
                          new_module)

    def test_package_creation(self):
        new_package = testutils.create_package(self.project, 'pkg')
        self.assertTrue(new_package.is_folder())
        self.assertEqual(self.project.get_resource('pkg'), new_package)
        self.assertEqual(self.project.get_resource('pkg/__init__.py'),
                          new_package.get_child('__init__.py'))

    def test_nested_package_creation(self):
        testutils.create_package(self.project, 'pkg1')
        nested_package = testutils.create_package(self.project, 'pkg1.pkg2')
        self.assertEqual(self.project.get_resource('pkg1/pkg2'),
                          nested_package)

    def test_packaged_package_creation_with_nested_src(self):
        src = self.project.root.create_folder('src')
        testutils.create_package(self.project, 'pkg1', src)
        nested_package = testutils.create_package(self.project, 'pkg1.pkg2',
                                                  src)
        self.assertEqual(self.project.get_resource('src/pkg1/pkg2'),
                          nested_package)

    def test_find_module(self):
        src = self.project.root.create_folder('src')
        samplemod = testutils.create_module(self.project, 'samplemod', src)
        found_module = self.project.find_module('samplemod')
        self.assertEqual(samplemod, found_module)

    def test_find_nested_module(self):
        src = self.project.root.create_folder('src')
        samplepkg = testutils.create_package(self.project, 'samplepkg', src)
        samplemod = testutils.create_module(self.project, 'samplemod',
                                            samplepkg)
        found_module = self.project.find_module('samplepkg.samplemod')
        self.assertEqual(samplemod, found_module)

    def test_find_multiple_module(self):
        src = self.project.root.create_folder('src')
        samplemod1 = testutils.create_module(self.project, 'samplemod', src)
        samplemod2 = testutils.create_module(self.project, 'samplemod')
        test = self.project.root.create_folder('test')
        samplemod3 = testutils.create_module(self.project, 'samplemod', test)
        found_module = self.project.find_module('samplemod')
        self.assertTrue(samplemod1 == found_module or
                        samplemod2 == found_module or
                        samplemod3 == found_module)

    def test_find_module_packages(self):
        src = self.project.root
        samplepkg = testutils.create_package(self.project, 'samplepkg', src)
        found_module = self.project.find_module('samplepkg')
        self.assertEqual(samplepkg, found_module)

    def test_find_module_when_module_and_package_with_the_same_name(self):
        src = self.project.root
        testutils.create_module(self.project, 'sample', src)
        samplepkg = testutils.create_package(self.project, 'sample', src)
        found_module = self.project.find_module('sample')
        self.assertEqual(samplepkg, found_module)

    def test_source_folders_preference(self):
        testutils.create_package(self.project, 'pkg1')
        testutils.create_package(self.project, 'pkg1.src2')
        lost = testutils.create_module(self.project, 'pkg1.src2.lost')
        self.assertEqual(self.project.find_module('lost'), None)
        self.project.close()
        from rope.base.project import Project
        self.project = Project(self.project.address,
                               source_folders=['pkg1/src2'])
        self.assertEqual(self.project.find_module('lost'), lost)

    def test_get_pyname_definition_location(self):
        mod = libutils.get_string_module(self.project, 'a_var = 20\n')
        a_var = mod['a_var']
        self.assertEqual((mod, 1), a_var.get_definition_location())

    def test_get_pyname_definition_location_functions(self):
        mod = libutils.get_string_module(
            self.project, 'def a_func():\n    pass\n')
        a_func = mod['a_func']
        self.assertEqual((mod, 1), a_func.get_definition_location())

    def test_get_pyname_definition_location_class(self):
        code = 'class AClass(object):\n    pass\n\n'
        mod = libutils.get_string_module(self.project, code)
        a_class = mod['AClass']
        self.assertEqual((mod, 1), a_class.get_definition_location())

    def test_get_pyname_definition_location_local_variables(self):
        mod = libutils.get_string_module(
            self.project, 'def a_func():\n    a_var = 10\n')
        a_func_scope = mod.get_scope().get_scopes()[0]
        a_var = a_func_scope['a_var']
        self.assertEqual((mod, 2), a_var.get_definition_location())

    def test_get_pyname_definition_location_reassigning(self):
        mod = libutils.get_string_module(
            self.project, 'a_var = 20\na_var=30\n')
        a_var = mod['a_var']
        self.assertEqual((mod, 1), a_var.get_definition_location())

    def test_get_pyname_definition_location_importes(self):
        testutils.create_module(self.project, 'mod')
        mod = libutils.get_string_module(self.project, 'import mod\n')
        imported_module = self.project.get_module('mod')
        module_pyname = mod['mod']
        self.assertEqual((imported_module, 1),
                          module_pyname.get_definition_location())

    def test_get_pyname_definition_location_imports(self):
        module_resource = testutils.create_module(self.project, 'mod')
        module_resource.write('\ndef a_func():\n    pass\n')
        imported_module = self.project.get_module('mod')
        mod = libutils.get_string_module(
            self.project, 'from mod import a_func\n')
        a_func = mod['a_func']
        self.assertEqual((imported_module, 2),
                          a_func.get_definition_location())

    def test_get_pyname_definition_location_parameters(self):
        code = 'def a_func(param1, param2):\n    a_var = param\n'
        mod = libutils.get_string_module(self.project, code)
        a_func_scope = mod.get_scope().get_scopes()[0]
        param1 = a_func_scope['param1']
        self.assertEqual((mod, 1), param1.get_definition_location())
        param2 = a_func_scope['param2']
        self.assertEqual((mod, 1), param2.get_definition_location())

    def test_module_get_resource(self):
        module_resource = testutils.create_module(self.project, 'mod')
        module = self.project.get_module('mod')
        self.assertEqual(module_resource, module.get_resource())
        string_module = libutils.get_string_module(
            self.project, 'from mod import a_func\n')
        self.assertEqual(None, string_module.get_resource())

    def test_get_pyname_definition_location_class2(self):
        code = 'class AClass(object):\n' \
               '    def __init__(self):\n' \
               '        self.an_attr = 10\n'
        mod = libutils.get_string_module(self.project, code)
        a_class = mod['AClass'].get_object()
        an_attr = a_class['an_attr']
        self.assertEqual((mod, 3), an_attr.get_definition_location())

    def test_import_not_found_module_get_definition_location(self):
        mod = libutils.get_string_module(
            self.project, 'import doesnotexist\n')
        does_not_exist = mod['doesnotexist']
        self.assertEqual((None, None),
                          does_not_exist.get_definition_location())

    def test_from_not_found_module_get_definition_location(self):
        mod = libutils.get_string_module(
            self.project, 'from doesnotexist import Sample\n')
        sample = mod['Sample']
        self.assertEqual((None, None), sample.get_definition_location())

    def test_from_package_import_module_get_definition_location(self):
        pkg = testutils.create_package(self.project, 'pkg')
        testutils.create_module(self.project, 'mod', pkg)
        pkg_mod = self.project.get_module('pkg.mod')
        mod = libutils.get_string_module(
            self.project, 'from pkg import mod\n')
        imported_mod = mod['mod']
        self.assertEqual((pkg_mod, 1),
                          imported_mod.get_definition_location())

    def test_get_module_for_defined_pyobjects(self):
        mod = libutils.get_string_module(
            self.project, 'class AClass(object):\n    pass\n')
        a_class = mod['AClass'].get_object()
        self.assertEqual(mod, a_class.get_module())

    def test_get_definition_location_for_packages(self):
        testutils.create_package(self.project, 'pkg')
        init_module = self.project.get_module('pkg.__init__')
        mod = libutils.get_string_module(self.project, 'import pkg\n')
        pkg_pyname = mod['pkg']
        self.assertEqual((init_module, 1),
                          pkg_pyname.get_definition_location())

    def test_get_definition_location_for_filtered_packages(self):
        pkg = testutils.create_package(self.project, 'pkg')
        testutils.create_module(self.project, 'mod', pkg)
        init_module = self.project.get_module('pkg.__init__')
        mod = libutils.get_string_module(self.project, 'import pkg.mod')
        pkg_pyname = mod['pkg']
        self.assertEqual((init_module, 1),
                          pkg_pyname.get_definition_location())

    def test_out_of_project_modules(self):
        scope = libutils.get_string_scope(
            self.project, 'import rope.base.project as project\n')
        imported_module = scope['project'].get_object()
        self.assertTrue('Project' in imported_module)

    def test_file_encoding_reading(self):
        contents = u'# -*- coding: utf-8 -*-\n' + \
            u'#\N{LATIN SMALL LETTER I WITH DIAERESIS}\n'
        mod = testutils.create_module(self.project, 'mod')
        mod.write(contents)
        self.project.get_module('mod')

    def test_global_keyword(self):
        contents = 'a_var = 1\ndef a_func():\n    global a_var\n'
        mod = libutils.get_string_module(self.project, contents)
        global_var = mod['a_var']
        func_scope = mod['a_func'].get_object().get_scope()
        local_var = func_scope['a_var']
        self.assertEqual(global_var, local_var)

    def test_not_leaking_for_vars_inside_parent_scope(self):
        mod = testutils.create_module(self.project, 'mod')
        code = 'class C(object):\n' \
               '    def f(self):\n' \
               '        for my_var1, my_var2 in []:\n' \
               '            pass\n'
        mod.write(code)
        pymod = self.pycore.resource_to_pyobject(mod)
        c_class = pymod['C'].get_object()
        self.assertFalse('my_var1' in c_class)
        self.assertFalse('my_var2' in c_class)

    def test_not_leaking_for_vars_inside_parent_scope2(self):
        mod = testutils.create_module(self.project, 'mod')
        code = 'class C(object):\n' \
               '    def f(self):\n' \
               '        for my_var in []:\n' \
               '            pass\n'
        mod.write(code)
        pymod = self.pycore.resource_to_pyobject(mod)
        c_class = pymod['C'].get_object()
        self.assertFalse('my_var' in c_class)

    def test_variables_defined_in_excepts(self):
        mod = testutils.create_module(self.project, 'mod')
        code = 'try:\n' \
               '    myvar1 = 1\n' \
               'except:\n' \
               '    myvar2 = 1\n' \
               'finally:\n' \
               '    myvar3 = 1\n'
        mod.write(code)
        pymod = self.pycore.resource_to_pyobject(mod)
        self.assertTrue('myvar1' in pymod)
        self.assertTrue('myvar2' in pymod)
        self.assertTrue('myvar3' in pymod)

    def test_not_leaking_tuple_assigned_names_inside_parent_scope(self):
        mod = testutils.create_module(self.project, 'mod')
        code = 'class C(object):\n' \
               '    def f(self):\n' \
               '        var1, var2 = range(2)\n'
        mod.write(code)
        pymod = self.pycore.resource_to_pyobject(mod)
        c_class = pymod['C'].get_object()
        self.assertFalse('var1' in c_class)

    @testutils.only_for('2.5')
    def test_with_statement_variables(self):
        code = 'import threading\nwith threading.lock() as var:    pass\n'
        if sys.version_info < (2, 6, 0):
            code = 'from __future__ import with_statement\n' + code
        pymod = libutils.get_string_module(self.project, code)
        self.assertTrue('var' in pymod)

    @testutils.only_for('2.5')
    def test_with_statement_variables_and_tuple_assignment(self):
        code = 'class A(object):\n' \
               '    def __enter__(self):' \
               '        return (1, 2)\n' \
               '    def __exit__(self, type, value, tb):\n' \
               '        pass\n'\
               'with A() as (a, b):\n' \
               '    pass\n'
        if sys.version_info < (2, 6, 0):
            code = 'from __future__ import with_statement\n' + code
        pymod = libutils.get_string_module(self.project, code)
        self.assertTrue('a' in pymod)
        self.assertTrue('b' in pymod)

    @testutils.only_for('2.5')
    def test_with_statement_variable_type(self):
        code = 'class A(object):\n' \
               '    def __enter__(self):\n' \
               '        return self\n'\
               '    def __exit__(self, type, value, tb):\n' \
               '        pass\n' \
               'with A() as var:\n' \
               '    pass\n'
        if sys.version_info < (2, 6, 0):
            code = 'from __future__ import with_statement\n' + code
        pymod = libutils.get_string_module(self.project, code)
        a_class = pymod['A'].get_object()
        var = pymod['var'].get_object()
        self.assertEqual(a_class, var.get_type())

    @testutils.only_for('2.7')
    def test_nested_with_statement_variable_type(self):
        code = 'class A(object):\n' \
               '    def __enter__(self):\n' \
               '        return self\n'\
               '    def __exit__(self, type, value, tb):\n' \
               '        pass\n' \
               'class B(object):\n' \
               '    def __enter__(self):\n' \
               '        return self\n'\
               '    def __exit__(self, type, value, tb):\n' \
               '        pass\n' \
               'with A() as var_a, B() as var_b:\n' \
               '    pass\n'
        if sys.version_info < (2, 6, 0):
            code = 'from __future__ import with_statement\n' + code
        pymod = libutils.get_string_module(self.project, code)
        a_class = pymod['A'].get_object()
        var_a = pymod['var_a'].get_object()
        self.assertEqual(a_class, var_a.get_type())

        b_class = pymod['B'].get_object()
        var_b = pymod['var_b'].get_object()
        self.assertEqual(b_class, var_b.get_type())

    @testutils.only_for('2.5')
    def test_with_statement_with_no_vars(self):
        code = 'with open("file"):    pass\n'
        if sys.version_info < (2, 6, 0):
            code = 'from __future__ import with_statement\n' + code
        pymod = libutils.get_string_module(self.project, code)
        pymod.get_attributes()

    def test_with_statement(self):
        code = 'a = 10\n' \
               'with open("file") as f:    pass\n'
        pymod = libutils.get_string_module(self.project, code)

        assigned = pymod.get_attribute('a')
        self.assertEqual(BuiltinClass, type(assigned.get_object().get_type()))

        assigned = pymod.get_attribute('f')
        self.assertEqual(File, type(assigned.get_object().get_type()))

    def test_check_for_else_block(self):
        code = 'for i in range(10):\n' \
               '    pass\n' \
               'else:\n' \
               '    myvar = 1\n'
        mod = libutils.get_string_module(self.project, code)
        a_var = mod['myvar']
        self.assertEqual((mod, 4), a_var.get_definition_location())

    def test_check_names_defined_in_whiles(self):
        mod = libutils.get_string_module(
            self.project, 'while False:\n    myvar = 1\n')
        a_var = mod['myvar']
        self.assertEqual((mod, 2), a_var.get_definition_location())

    def test_get_definition_location_in_tuple_assnames(self):
        mod = libutils.get_string_module(
            self.project, 'def f(x):\n    x.z, a = range(2)\n')
        x = mod['f'].get_object().get_scope()['x']
        a = mod['f'].get_object().get_scope()['a']
        self.assertEqual((mod, 1), x.get_definition_location())
        self.assertEqual((mod, 2), a.get_definition_location())

    def test_syntax_errors_in_code(self):
        with self.assertRaises(exceptions.ModuleSyntaxError):
            libutils.get_string_module(self.project, 'xyx print\n')

    def test_holding_error_location_information(self):
        try:
            libutils.get_string_module(self.project, 'xyx print\n')
        except exceptions.ModuleSyntaxError as e:
            self.assertEqual(1, e.lineno)

    def test_no_exceptions_on_module_encoding_problems(self):
        mod = testutils.create_module(self.project, 'mod')
        contents = b'\nsdsdsd\n\xa9\n'
        file = open(mod.real_path, 'wb')
        file.write(contents)
        file.close()
        mod.read()

    def test_syntax_errors_when_cannot_decode_file2(self):
        mod = testutils.create_module(self.project, 'mod')
        contents = b'\n\xa9\n'
        file = open(mod.real_path, 'wb')
        file.write(contents)
        file.close()
        with self.assertRaises(exceptions.ModuleSyntaxError):
            self.pycore.resource_to_pyobject(mod)

    def test_syntax_errors_when_null_bytes(self):
        mod = testutils.create_module(self.project, 'mod')
        contents = b'\n\x00\n'
        file = open(mod.real_path, 'wb')
        file.write(contents)
        file.close()
        with self.assertRaises(exceptions.ModuleSyntaxError):
            self.pycore.resource_to_pyobject(mod)

    def test_syntax_errors_when_bad_strs(self):
        mod = testutils.create_module(self.project, 'mod')
        contents = b'\n"\\x0"\n'
        file = open(mod.real_path, 'wb')
        file.write(contents)
        file.close()
        with self.assertRaises(exceptions.ModuleSyntaxError):
            self.pycore.resource_to_pyobject(mod)

    def test_not_reaching_maximum_recursions_with_from_star_imports(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('from mod2 import *\n')
        mod2.write('from mod1 import *\n')
        pymod1 = self.pycore.resource_to_pyobject(mod1)
        pymod1.get_attributes()

    def test_not_reaching_maximum_recursions_when_importing_variables(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('from mod2 import myvar\n')
        mod2.write('from mod1 import myvar\n')
        pymod1 = self.pycore.resource_to_pyobject(mod1)
        pymod1['myvar'].get_object()

    def test_not_reaching_maximum_recursions_when_importing_variables2(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('from mod1 import myvar\n')
        pymod1 = self.pycore.resource_to_pyobject(mod1)
        pymod1['myvar'].get_object()

    def test_pyobject_equality_should_compare_types(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod1.write('var1 = ""\nvar2 = ""\n')
        pymod1 = self.pycore.resource_to_pyobject(mod1)
        self.assertEqual(pymod1['var1'].get_object(),
                          pymod1['var2'].get_object())


class PyCoreInProjectsTest(unittest.TestCase):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.project = testutils.sample_project()
        self.pycore = self.project.pycore
        samplemod = testutils.create_module(self.project, 'samplemod')
        code = 'class SampleClass(object):\n' \
               '    def sample_method():\n' \
               '        pass\n\n' \
               'def sample_func():\n' \
               '    pass\n' \
               'sample_var = 10\n\n' \
               'def _underlined_func():\n' \
               '    pass\n\n'
        samplemod.write(code)
        package = testutils.create_package(self.project, 'package')
        testutils.create_module(self.project, 'nestedmod', package)

    def tearDown(self):
        testutils.remove_project(self.project)
        super(self.__class__, self).tearDown()

    def test_simple_import(self):
        mod = libutils.get_string_module(
            self.project, 'import samplemod\n')
        samplemod = mod['samplemod'].get_object()
        self.assertEqual(get_base_type('Module'), samplemod.get_type())

    def test_from_import_class(self):
        mod = libutils.get_string_module(
            self.project, 'from samplemod import SampleClass\n')
        result = mod['SampleClass'].get_object()
        self.assertEqual(get_base_type('Type'), result.get_type())
        self.assertTrue('sample_func' not in mod.get_attributes())

    def test_from_import_star(self):
        mod = libutils.get_string_module(
            self.project, 'from samplemod import *\n')
        self.assertEqual(get_base_type('Type'),
                          mod['SampleClass'].get_object().get_type())
        self.assertEqual(get_base_type('Function'),
                          mod['sample_func'].get_object().get_type())
        self.assertTrue(mod['sample_var'] is not None)

    def test_from_import_star_overwriting(self):
        code = 'from samplemod import *\n' \
               'class SampleClass(object):\n    pass\n'
        mod = libutils.get_string_module(self.project, code)
        samplemod = self.project.get_module('samplemod')
        sample_class = samplemod['SampleClass'].get_object()
        self.assertNotEqual(sample_class,
                             mod.get_attributes()['SampleClass'].get_object())

    def test_from_import_star_not_imporing_underlined(self):
        mod = libutils.get_string_module(
            self.project, 'from samplemod import *')
        self.assertTrue('_underlined_func' not in mod.get_attributes())

    def test_from_import_star_imports_in_functions(self):
        mod = libutils.get_string_module(
            self.project, 'def f():\n    from os import *\n')
        mod['f'].get_object().get_scope().get_names()

    def test_from_package_import_mod(self):
        mod = libutils.get_string_module(
            self.project, 'from package import nestedmod\n')
        self.assertEqual(get_base_type('Module'),
                          mod['nestedmod'].get_object().get_type())

    # XXX: Deciding to import everything on import start from packages
    def xxx_test_from_package_import_star(self):
        mod = libutils.get_string_module(
            self.project, 'from package import *\n')
        self.assertTrue('nestedmod' not in mod.get_attributes())

    def test_unknown_when_module_cannot_be_found(self):
        mod = libutils.get_string_module(
            self.project, 'from doesnotexist import nestedmod\n')
        self.assertTrue('nestedmod' in mod)

    def test_from_import_function(self):
        code = 'def f():\n    from samplemod import SampleClass\n'
        scope = libutils.get_string_scope(self.project, code)
        self.assertEqual(get_base_type('Type'),
                          scope.get_scopes()[0]['SampleClass'].
                          get_object().get_type())

    def test_circular_imports(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('import mod2\n')
        mod2.write('import mod1\n')
        self.project.get_module('mod1')

    def test_circular_imports2(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write(
            'from mod2 import Sample2\nclass Sample1(object):\n    pass\n')
        mod2.write(
            'from mod1 import Sample1\nclass Sample2(object):\n    pass\n')
        self.project.get_module('mod1').get_attributes()

    def test_multi_dot_imports(self):
        pkg = testutils.create_package(self.project, 'pkg')
        pkg_mod = testutils.create_module(self.project, 'mod', pkg)
        pkg_mod.write('def sample_func():\n    pass\n')
        mod = libutils.get_string_module(self.project, 'import pkg.mod\n')
        self.assertTrue('pkg' in mod)
        self.assertTrue('sample_func' in mod['pkg'].get_object()['mod'].
                        get_object())

    def test_multi_dot_imports2(self):
        pkg = testutils.create_package(self.project, 'pkg')
        testutils.create_module(self.project, 'mod1', pkg)
        testutils.create_module(self.project, 'mod2', pkg)
        mod = libutils.get_string_module(
            self.project, 'import pkg.mod1\nimport pkg.mod2\n')
        package = mod['pkg'].get_object()
        self.assertEqual(2, len(package.get_attributes()))
        self.assertTrue('mod1' in package and
                        'mod2' in package)

    def test_multi_dot_imports3(self):
        pkg1 = testutils.create_package(self.project, 'pkg1')
        pkg2 = testutils.create_package(self.project, 'pkg2', pkg1)
        testutils.create_module(self.project, 'mod1', pkg2)
        testutils.create_module(self.project, 'mod2', pkg2)
        code = 'import pkg1.pkg2.mod1\nimport pkg1.pkg2.mod2\n'
        mod = libutils.get_string_module(self.project, code)
        package1 = mod['pkg1'].get_object()
        package2 = package1['pkg2'].get_object()
        self.assertEqual(2, len(package2.get_attributes()))
        self.assertTrue('mod1' in package2 and 'mod2' in package2)

    def test_multi_dot_imports_as(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod1 = testutils.create_module(self.project, 'mod1', pkg)
        mod1.write('def f():\n    pass\n')
        mod = libutils.get_string_module(
            self.project, 'import pkg.mod1 as mod1\n')
        module = mod['mod1'].get_object()
        self.assertTrue('f' in module)

    # TODO: not showing unimported names as attributes of packages
    def xxx_test_from_package_import_package(self):
        pkg1 = testutils.create_package(self.project, 'pkg1')
        pkg2 = testutils.create_package(self.project, 'pkg2', pkg1)
        testutils.create_module(self.project, 'mod', pkg2)
        mod = libutils.get_string_module(
            self.project, 'from pkg1 import pkg2\n')
        package = mod['pkg2']
        self.assertEqual(0, len(package.get_attributes()))

    def test_invalidating_cache_after_resource_change(self):
        module = testutils.create_module(self.project, 'mod')
        module.write('import sys\n')
        mod1 = self.project.get_module('mod')
        self.assertTrue('var' not in mod1.get_attributes())
        module.write('var = 10\n')
        mod2 = self.project.get_module('mod')
        self.assertTrue('var' in mod2)

    def test_invalidating_cache_after_resource_change_for_init_dot_pys(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod = testutils.create_module(self.project, 'mod')
        init_dot_py = pkg.get_child('__init__.py')
        init_dot_py.write('a_var = 10\n')
        mod.write('import pkg\n')
        pymod = self.project.get_module('mod')
        self.assertTrue('a_var' in pymod['pkg'].get_object())
        init_dot_py.write('new_var = 10\n')
        self.assertTrue('a_var' not in
                        pymod['pkg'].get_object().get_attributes())

    def test_invalidating_cache_after_rsrc_chng_for_nested_init_dot_pys(self):
        pkg1 = testutils.create_package(self.project, 'pkg1')
        pkg2 = testutils.create_package(self.project, 'pkg2', pkg1)
        mod = testutils.create_module(self.project, 'mod')
        init_dot_py = pkg2.get_child('__init__.py')
        init_dot_py.write('a_var = 10\n')
        mod.write('import pkg1\n')
        pymod = self.project.get_module('mod')
        self.assertTrue('a_var' in
                        pymod['pkg1'].get_object()['pkg2'].get_object())
        init_dot_py.write('new_var = 10\n')
        self.assertTrue('a_var' not in
                        pymod['pkg1'].get_object()['pkg2'].get_object())

    def test_from_import_nonexistent_module(self):
        code = 'from doesnotexistmod import DoesNotExistClass\n'
        mod = libutils.get_string_module(self.project, code)
        self.assertTrue('DoesNotExistClass' in mod)
        self.assertEqual(get_base_type('Unknown'),
                          mod['DoesNotExistClass'].
                          get_object().get_type())

    def test_from_import_nonexistent_name(self):
        code = 'from samplemod import DoesNotExistClass\n'
        mod = libutils.get_string_module(self.project, code)
        self.assertTrue('DoesNotExistClass' in mod)
        self.assertEqual(get_base_type('Unknown'),
                          mod['DoesNotExistClass'].
                          get_object().get_type())

    def test_not_considering_imported_names_as_sub_scopes(self):
        code = 'from samplemod import SampleClass\n'
        scope = libutils.get_string_scope(self.project, code)
        self.assertEqual(0, len(scope.get_scopes()))

    def test_not_considering_imported_modules_as_sub_scopes(self):
        scope = libutils.get_string_scope(
            self.project, 'import samplemod\n')
        self.assertEqual(0, len(scope.get_scopes()))

    def test_inheriting_dotted_base_class(self):
        code = 'import samplemod\n' \
               'class Derived(samplemod.SampleClass):\n' \
               '    pass\n'
        mod = libutils.get_string_module(self.project, code)
        derived = mod['Derived'].get_object()
        self.assertTrue('sample_method' in derived)

    def test_self_in_methods(self):
        code = 'class Sample(object):\n' \
               '    def func(self):\n' \
               '        pass\n'
        scope = libutils.get_string_scope(self.project, code)
        sample_class = scope['Sample'].get_object()
        func_scope = scope.get_scopes()[0].get_scopes()[0]
        self.assertEqual(sample_class,
                          func_scope['self'].get_object().get_type())
        self.assertTrue('func' in func_scope['self'].get_object())

    def test_none_assignments_in_classes(self):
        code = 'class C(object):\n' \
               '    var = ""\n' \
               '    def f(self):\n' \
               '        self.var += "".join([])\n'
        scope = libutils.get_string_scope(self.project, code)
        c_class = scope['C'].get_object()
        self.assertTrue('var' in c_class)

    def test_self_in_methods_with_decorators(self):
        code = 'class Sample(object):\n' \
               '    @staticmethod\n' \
               '    def func(self):\n' \
               '        pass\n'
        scope = libutils.get_string_scope(self.project, code)
        sample_class = scope['Sample'].get_object()
        func_scope = scope.get_scopes()[0].get_scopes()[0]
        self.assertNotEqual(sample_class,
                             func_scope['self'].get_object().get_type())

    def test_location_of_imports_when_importing(self):
        mod = testutils.create_module(self.project, 'mod')
        mod.write('from samplemod import SampleClass\n')
        scope = libutils.get_string_scope(
            self.project, 'from mod import SampleClass\n')
        sample_class = scope['SampleClass']
        samplemod = self.project.get_module('samplemod')
        self.assertEqual((samplemod, 1),
                          sample_class.get_definition_location())

    def test_nested_modules(self):
        pkg = testutils.create_package(self.project, 'pkg')
        testutils.create_module(self.project, 'mod', pkg)
        imported_module = self.project.get_module('pkg.mod')
        scope = libutils.get_string_scope(self.project, 'import pkg.mod\n')
        mod_pyobject = scope['pkg'].get_object()['mod']
        self.assertEqual((imported_module, 1),
                          mod_pyobject.get_definition_location())

    def test_reading_init_dot_py(self):
        pkg = testutils.create_package(self.project, 'pkg')
        init_dot_py = pkg.get_child('__init__.py')
        init_dot_py.write('a_var = 1\n')
        pkg_object = self.project.get_module('pkg')
        self.assertTrue('a_var' in pkg_object)

    def test_relative_imports(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod1 = testutils.create_module(self.project, 'mod1', pkg)
        mod2 = testutils.create_module(self.project, 'mod2', pkg)
        mod2.write('import mod1\n')
        mod1_object = self.pycore.resource_to_pyobject(mod1)
        mod2_object = self.pycore.resource_to_pyobject(mod2)
        self.assertEqual(mod1_object,
                          mod2_object.get_attributes()['mod1'].get_object())

    def test_relative_froms(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod1 = testutils.create_module(self.project, 'mod1', pkg)
        mod2 = testutils.create_module(self.project, 'mod2', pkg)
        mod1.write('def a_func():\n    pass\n')
        mod2.write('from mod1 import a_func\n')
        mod1_object = self.pycore.resource_to_pyobject(mod1)
        mod2_object = self.pycore.resource_to_pyobject(mod2)
        self.assertEqual(mod1_object['a_func'].get_object(),
                          mod2_object['a_func'].get_object())

    def test_relative_imports_for_string_modules(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod1 = testutils.create_module(self.project, 'mod1', pkg)
        mod2 = testutils.create_module(self.project, 'mod2', pkg)
        mod2.write('import mod1\n')
        mod1_object = self.pycore.resource_to_pyobject(mod1)
        mod2_object = libutils.get_string_module(
            self.project, mod2.read(), mod2)
        self.assertEqual(mod1_object, mod2_object['mod1'].get_object())

    def test_relative_imports_for_string_scopes(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod1 = testutils.create_module(self.project, 'mod1', pkg)
        mod2 = testutils.create_module(self.project, 'mod2', pkg)
        mod2.write('import mod1\n')
        mod1_object = self.pycore.resource_to_pyobject(mod1)
        mod2_scope = libutils.get_string_scope(self.project, mod2.read(),
                                               mod2)
        self.assertEqual(mod1_object, mod2_scope['mod1'].get_object())

    @testutils.only_for('2.5')
    def test_new_style_relative_imports(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod1 = testutils.create_module(self.project, 'mod1', pkg)
        mod2 = testutils.create_module(self.project, 'mod2', pkg)
        mod2.write('from . import mod1\n')
        mod1_object = self.pycore.resource_to_pyobject(mod1)
        mod2_object = self.pycore.resource_to_pyobject(mod2)
        self.assertEqual(mod1_object, mod2_object['mod1'].get_object())

    @testutils.only_for('2.5')
    def test_new_style_relative_imports2(self):
        pkg = testutils.create_package(self.project, 'pkg')
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2', pkg)
        mod1.write('def a_func():\n    pass\n')
        mod2.write('from ..mod1 import a_func\n')
        mod1_object = self.pycore.resource_to_pyobject(mod1)
        mod2_object = self.pycore.resource_to_pyobject(mod2)
        self.assertEqual(mod1_object['a_func'].get_object(),
                          mod2_object['a_func'].get_object())

    def test_invalidating_cache_for_from_imports_after_resource_change(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod2.write('def a_func():\n    print(1)\n')
        mod1.write('from mod2 import a_func\na_func()\n')

        pymod1 = self.project.get_module('mod1')
        pymod2 = self.project.get_module('mod2')
        self.assertEqual(pymod1['a_func'].get_object(),
                          pymod2['a_func'].get_object())
        mod2.write(mod2.read() + '\n')
        pymod2 = self.project.get_module('mod2')
        self.assertEqual(pymod1['a_func'].get_object(),
                          pymod2['a_func'].get_object())

    def test_invalidating_superclasses_after_change(self):
        mod1 = testutils.create_module(self.project, 'mod1')
        mod2 = testutils.create_module(self.project, 'mod2')
        mod1.write('class A(object):\n    def func1(self):\n        pass\n')
        mod2.write('import mod1\nclass B(mod1.A):\n    pass\n')

        b_class = self.project.get_module('mod2')['B'].get_object()
        self.assertTrue('func1' in b_class)

        mod1.write('class A(object):\n    def func2(self):\n        pass\n')
        self.assertTrue('func2' in b_class)

    def test_caching_pymodule_with_syntax_errors(self):
        self.project.prefs['ignore_syntax_errors'] = True
        self.project.prefs['automatic_soa'] = True
        self.project.pycore._init_automatic_soa()
        source = 'import sys\nab cd'
        mod = testutils.create_module(self.project, 'mod')
        mod.write(source)
        from rope.contrib import fixsyntax
        fixer = fixsyntax.FixSyntax(self.project, source, mod, 10)
        pymodule = fixer.get_pymodule()
        self.assertTrue(pymodule.source_code.startswith('import sys\npass\n'))


class TextChangeDetectorTest(unittest.TestCase):

    def test_trivial_case(self):
        detector = _TextChangeDetector('\n', '\n')
        self.assertFalse(detector.is_changed(1, 1))

    def test_one_line_change(self):
        detector = _TextChangeDetector('1\n2\n', '1\n3\n')
        self.assertFalse(detector.is_changed(1, 1))
        self.assertTrue(detector.is_changed(2, 2))

    def test_line_expansion(self):
        detector = _TextChangeDetector('1\n2\n', '1\n3\n4\n2\n')
        self.assertFalse(detector.is_changed(1, 1))
        self.assertFalse(detector.is_changed(2, 2))

    def test_line_removals(self):
        detector = _TextChangeDetector('1\n3\n4\n2\n', '1\n2\n')
        self.assertFalse(detector.is_changed(1, 1))
        self.assertTrue(detector.is_changed(2, 3))
        self.assertFalse(detector.is_changed(4, 4))

    def test_multi_line_checks(self):
        detector = _TextChangeDetector('1\n2\n', '1\n3\n')
        self.assertTrue(detector.is_changed(1, 2))

    def test_consume_change(self):
        detector = _TextChangeDetector('1\n2\n', '1\n3\n')
        self.assertTrue(detector.is_changed(1, 2))
        self.assertTrue(detector.consume_changes(1, 2))
        self.assertFalse(detector.is_changed(1, 2))


class PyCoreProjectConfigsTest(unittest.TestCase):

    def setUp(self):
        super(PyCoreProjectConfigsTest, self).setUp()
        self.project = None

    def tearDown(self):
        if self.project:
            testutils.remove_project(self.project)
        super(PyCoreProjectConfigsTest, self).tearDown()

    def test_python_files_config(self):
        self.project = testutils.sample_project(python_files=['myscript'])
        myscript = self.project.root.create_file('myscript')
        self.assertTrue(self.project.pycore.is_python_file(myscript))

    def test_ignore_bad_imports(self):
        self.project = testutils.sample_project(ignore_bad_imports=True)
        pymod = libutils.get_string_module(
            self.project, 'import some_nonexistent_module\n')
        self.assertFalse('some_nonexistent_module' in pymod)

    def test_ignore_bad_imports_for_froms(self):
        self.project = testutils.sample_project(ignore_bad_imports=True)
        pymod = libutils.get_string_module(
            self.project, 'from some_nonexistent_module import var\n')
        self.assertFalse('var' in pymod)

    def test_reporting_syntax_errors_with_force_errors(self):
        self.project = testutils.sample_project(ignore_syntax_errors=True)
        mod = testutils.create_module(self.project, 'mod')
        mod.write('syntax error ...\n')
        with self.assertRaises(exceptions.ModuleSyntaxError):
            self.project.pycore.resource_to_pyobject(mod, force_errors=True)

    def test_reporting_syntax_errors_in_strings_with_force_errors(self):
        self.project = testutils.sample_project(ignore_syntax_errors=True)
        with self.assertRaises(exceptions.ModuleSyntaxError):
            libutils.get_string_module(
                self.project, 'syntax error ...', force_errors=True)

    def test_not_raising_errors_for_strings_with_ignore_errors(self):
        self.project = testutils.sample_project(ignore_syntax_errors=True)
        libutils.get_string_module(self.project, 'syntax error ...')

    def test_reporting_syntax_errors_with_force_errors_for_packages(self):
        self.project = testutils.sample_project(ignore_syntax_errors=True)
        pkg = testutils.create_package(self.project, 'pkg')
        pkg.get_child('__init__.py').write('syntax error ...\n')
        with self.assertRaises(exceptions.ModuleSyntaxError):
            self.project.pycore.resource_to_pyobject(pkg, force_errors=True)


def suite():
    result = unittest.TestSuite()
    result.addTests(unittest.makeSuite(PyCoreTest))
    result.addTests(unittest.makeSuite(PyCoreInProjectsTest))
    result.addTests(unittest.makeSuite(TextChangeDetectorTest))
    result.addTests(unittest.makeSuite(PyCoreProjectConfigsTest))
    return result


if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

from rope.base import exceptions
from ropetest import testutils
from rope.refactor.usefunction import UseFunction


class UseFunctionTest(unittest.TestCase):

    def setUp(self):
        super(UseFunctionTest, self).setUp()
        self.project = testutils.sample_project()
        self.mod1 = testutils.create_module(self.project, 'mod1')
        self.mod2 = testutils.create_module(self.project, 'mod2')

    def tearDown(self):
        testutils.remove_project(self.project)
        super(UseFunctionTest, self).tearDown()

    def test_simple_case(self):
        code = 'def f():\n    pass\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.project.do(user.get_changes())
        self.assertEqual(code, self.mod1.read())

    def test_simple_function(self):
        code = 'def f(p):\n    print(p)\nprint(1)\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.project.do(user.get_changes())
        self.assertEqual('def f(p):\n    print(p)\nf(1)\n',
                          self.mod1.read())

    def test_simple_function2(self):
        code = 'def f(p):\n    print(p + 1)\nprint(1 + 1)\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.project.do(user.get_changes())
        self.assertEqual('def f(p):\n    print(p + 1)\nf(1)\n',
                          self.mod1.read())

    def test_functions_with_multiple_statements(self):
        code = 'def f(p):\n    r = p + 1\n    print(r)\nr = 2 + 1\nprint(r)\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.project.do(user.get_changes())
        self.assertEqual('def f(p):\n    r = p + 1\n    print(r)\nf(2)\n',
                          self.mod1.read())

    def test_returning(self):
        code = 'def f(p):\n    return p + 1\nr = 2 + 1\nprint(r)\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.project.do(user.get_changes())
        self.assertEqual(
            'def f(p):\n    return p + 1\nr = f(2)\nprint(r)\n',
            self.mod1.read())

    def test_returning_a_single_expression(self):
        code = 'def f(p):\n    return p + 1\nprint(2 + 1)\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.project.do(user.get_changes())
        self.assertEqual(
            'def f(p):\n    return p + 1\nprint(f(2))\n',
            self.mod1.read())

    def test_occurrences_in_other_modules(self):
        code = 'def f(p):\n    return p + 1\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.mod2.write('print(2 + 1)\n')
        self.project.do(user.get_changes())
        self.assertEqual('import mod1\nprint(mod1.f(2))\n',
                          self.mod2.read())

    def test_when_performing_on_non_functions(self):
        code = 'var = 1\n'
        self.mod1.write(code)
        with self.assertRaises(exceptions.RefactoringError):
            UseFunction(self.project, self.mod1, code.rindex('var'))

    def test_differing_in_the_inner_temp_names(self):
        code = 'def f(p):\n    a = p + 1\n    print(a)\nb = 2 + 1\nprint(b)\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.project.do(user.get_changes())
        self.assertEqual('def f(p):\n    a = p + 1\n    print(a)\nf(2)\n',
                          self.mod1.read())

    # TODO: probably new options should be added to restructure
    def xxx_test_being_a_bit_more_intelligent_when_returning_assigneds(self):
        code = 'def f(p):\n    a = p + 1\n    return a\n'\
               'var = 2 + 1\nprint(var)\n'
        self.mod1.write(code)
        user = UseFunction(self.project, self.mod1, code.rindex('f'))
        self.project.do(user.get_changes())
        self.assertEqual('def f(p):\n    a = p + 1\n    return a\n'
                          'var = f(p)\nprint(var)\n', self.mod1.read())

    def test_exception_when_performing_a_function_with_yield(self):
        code = 'def func():\n    yield 1\n'
        self.mod1.write(code)
        with self.assertRaises(exceptions.RefactoringError):
            UseFunction(self.project, self.mod1, code.index('func'))

    def test_exception_when_performing_a_function_two_returns(self):
        code = 'def func():\n    return 1\n    return 2\n'
        self.mod1.write(code)
        with self.assertRaises(exceptions.RefactoringError):
            UseFunction(self.project, self.mod1, code.index('func'))

    def test_exception_when_returns_is_not_the_last_statement(self):
        code = 'def func():\n    return 2\n    a = 1\n'
        self.mod1.write(code)
        with self.assertRaises(exceptions.RefactoringError):
            UseFunction(self.project, self.mod1, code.index('func'))


if __name__ == '__main__':
    unittest.main()



from rope.base import exceptions, change, taskhandle


class History(object):
    """A class that holds project history"""

    def __init__(self, project, maxundos=None):
        self.project = project
        self._undo_list = []
        self._redo_list = []
        self._maxundos = maxundos
        self._load_history()
        self.project.data_files.add_write_hook(self.write)
        self.current_change = None

    def _load_history(self):
        if self.save:
            result = self.project.data_files.read_data(
                'history', compress=self.compress, import_=True)
            if result is not None:
                to_change = change.DataToChange(self.project)
                for data in result[0]:
                    self._undo_list.append(to_change(data))
                for data in result[1]:
                    self._redo_list.append(to_change(data))

    def do(self, changes, task_handle=taskhandle.NullTaskHandle()):
        """Perform the change and add it to the `self.undo_list`

        Note that uninteresting changes (changes to ignored files)
        will not be appended to `self.undo_list`.

        """
        try:
            self.current_change = changes
            changes.do(change.create_job_set(task_handle, changes))
        finally:
            self.current_change = None
        if self._is_change_interesting(changes):
            self.undo_list.append(changes)
            self._remove_extra_items()
        del self.redo_list[:]

    def _remove_extra_items(self):
        if len(self.undo_list) > self.max_undos:
            del self.undo_list[0:len(self.undo_list) - self.max_undos]

    def _is_change_interesting(self, changes):
        for resource in changes.get_changed_resources():
            if not self.project.is_ignored(resource):
                return True
        return False

    def undo(self, change=None, drop=False,
             task_handle=taskhandle.NullTaskHandle()):
        """Redo done changes from the history

        When `change` is `None`, the last done change will be undone.
        If change is not `None` it should be an item from
        `self.undo_list`; this change and all changes that depend on
        it will be undone.  In both cases the list of undone changes
        will be returned.

        If `drop` is `True`, the undone change will not be appended to
        the redo list.

        """
        if not self._undo_list:
            raise exceptions.HistoryError('Undo list is empty')
        if change is None:
            change = self.undo_list[-1]
        dependencies = self._find_dependencies(self.undo_list, change)
        self._move_front(self.undo_list, dependencies)
        self._perform_undos(len(dependencies), task_handle)
        result = self.redo_list[-len(dependencies):]
        if drop:
            del self.redo_list[-len(dependencies):]
        return result

    def redo(self, change=None, task_handle=taskhandle.NullTaskHandle()):
        """Redo undone changes from the history

        When `change` is `None`, the last undone change will be
        redone.  If change is not `None` it should be an item from
        `self.redo_list`; this change and all changes that depend on
        it will be redone.  In both cases the list of redone changes
        will be returned.

        """
        if not self.redo_list:
            raise exceptions.HistoryError('Redo list is empty')
        if change is None:
            change = self.redo_list[-1]
        dependencies = self._find_dependencies(self.redo_list, change)
        self._move_front(self.redo_list, dependencies)
        self._perform_redos(len(dependencies), task_handle)
        return self.undo_list[-len(dependencies):]

    def _move_front(self, change_list, changes):
        for change in changes:
            change_list.remove(change)
            change_list.append(change)

    def _find_dependencies(self, change_list, change):
        index = change_list.index(change)
        return _FindChangeDependencies(change_list[index:])()

    def _perform_undos(self, count, task_handle):
        for i in range(count):
            self.current_change = self.undo_list[-1]
            try:
                job_set = change.create_job_set(task_handle,
                                                self.current_change)
                self.current_change.undo(job_set)
            finally:
                self.current_change = None
            self.redo_list.append(self.undo_list.pop())

    def _perform_redos(self, count, task_handle):
        for i in range(count):
            self.current_change = self.redo_list[-1]
            try:
                job_set = change.create_job_set(task_handle,
                                                self.current_change)
                self.current_change.do(job_set)
            finally:
                self.current_change = None
            self.undo_list.append(self.redo_list.pop())

    def contents_before_current_change(self, file):
        if self.current_change is None:
            return None
        result = self._search_for_change_contents([self.current_change], file)
        if result is not None:
            return result
        if file.exists() and not file.is_folder():
            return file.read()
        else:
            return None

    def _search_for_change_contents(self, change_list, file):
        for change_ in reversed(change_list):
            if isinstance(change_, change.ChangeSet):
                result = self._search_for_change_contents(change_.changes,
                                                          file)
                if result is not None:
                    return result
            if isinstance(change_, change.ChangeContents) and \
               change_.resource == file:
                return change_.old_contents

    def write(self):
        if self.save:
            data = []
            to_data = change.ChangeToData()
            self._remove_extra_items()
            data.append([to_data(change_) for change_ in self.undo_list])
            data.append([to_data(change_) for change_ in self.redo_list])
            self.project.data_files.write_data('history', data,
                                               compress=self.compress)

    def get_file_undo_list(self, resource):
        result = []
        for change in self.undo_list:
            if resource in change.get_changed_resources():
                result.append(change)
        return result

    def __str__(self):
        return 'History holds %s changes in memory' % \
               (len(self.undo_list) + len(self.redo_list))

    undo_list = property(lambda self: self._undo_list)
    redo_list = property(lambda self: self._redo_list)

    @property
    def tobe_undone(self):
        """The last done change if available, `None` otherwise"""
        if self.undo_list:
            return self.undo_list[-1]

    @property
    def tobe_redone(self):
        """The last undone change if available, `None` otherwise"""
        if self.redo_list:
            return self.redo_list[-1]

    @property
    def max_undos(self):
        if self._maxundos is None:
            return self.project.prefs.get('max_history_items', 100)
        else:
            return self._maxundos

    @property
    def save(self):
        return self.project.prefs.get('save_history', False)

    @property
    def compress(self):
        return self.project.prefs.get('compress_history', False)

    def clear(self):
        """Forget all undo and redo information"""
        del self.undo_list[:]
        del self.redo_list[:]


class _FindChangeDependencies(object):

    def __init__(self, change_list):
        self.change = change_list[0]
        self.change_list = change_list
        self.changed_resources = set(self.change.get_changed_resources())

    def __call__(self):
        result = [self.change]
        for change in self.change_list[1:]:
            if self._depends_on(change, result):
                result.append(change)
                self.changed_resources.update(change.get_changed_resources())
        return result

    def _depends_on(self, changes, result):
        for resource in changes.get_changed_resources():
            if resource is None:
                continue
            if resource in self.changed_resources:
                return True
            for changed in self.changed_resources:
                if resource.is_folder() and resource.contains(changed):
                    return True
                if changed.is_folder() and changed.contains(resource):
                    return True
        return False



try:
    import unittest2 as unittest
except ImportError:
    import unittest


from ropetest import testutils
from rope.contrib.fixmodnames import FixModuleNames
from rope.contrib.generate import create_module, create_package


# HACK: for making this test work on case-insensitive file-systems, it
# uses a name.replace('x', '_') fixer.
class FixModuleNamesTest(unittest.TestCase):

    def setUp(self):
        super(FixModuleNamesTest, self).setUp()
        self.project = testutils.sample_project()

    def tearDown(self):
        testutils.remove_project(self.project)
        super(FixModuleNamesTest, self).tearDown()

    def test_simple_module_renaming(self):
        mod = create_module(self.project, 'xod')
        self.project.do(FixModuleNames(self.project).get_changes(_fixer))
        self.assertFalse(mod.exists())
        self.assertTrue(self.project.get_resource('_od.py').exists())

    def test_packages_module_renaming(self):
        pkg = create_package(self.project, 'xkg')
        self.project.do(FixModuleNames(self.project).get_changes(_fixer))
        self.assertFalse(pkg.exists())
        self.assertTrue(self.project.get_resource('_kg/__init__.py').exists())

    def test_fixing_contents(self):
        mod1 = create_module(self.project, 'xod1')
        mod2 = create_module(self.project, 'xod2')
        mod1.write('import xod2\n')
        mod2.write('import xod1\n')
        self.project.do(FixModuleNames(self.project).get_changes(_fixer))
        newmod1 = self.project.get_resource('_od1.py')
        newmod2 = self.project.get_resource('_od2.py')
        self.assertEqual('import _od2\n', newmod1.read())
        self.assertEqual('import _od1\n', newmod2.read())

    def test_handling_nested_modules(self):
        pkg = create_package(self.project, 'xkg')
        mod = create_module(self.project, 'xkg.xod')  # noqa
        self.project.do(FixModuleNames(self.project).get_changes(_fixer))
        self.assertFalse(pkg.exists())
        self.assertTrue(self.project.get_resource('_kg/__init__.py').exists())
        self.assertTrue(self.project.get_resource('_kg/_od.py').exists())


def _fixer(name):
    return name.replace('x', '_')

if __name__ == '__main__':
    unittest.main()
